<?php
if (!defined('WPINC')) exit;

// This key is used to save and retrieve settings option from wp

/**
 * This class handles the communications between wpdm and the mailchimp api server
 *
 * It includes the mailchimp api v3
 *
 */

class WPDM_MailChimp_API
{

    /**
     * Singleton instance of the class
     *
     */
    static private $instance;

    /**
     * Mailchimp API Object
     *
     * This will attach to the Mailchimp API Object
     * only when a valid API key is provided and connected with the Mailchimp Server
     */
    private $api;

    /**
     * Is connected to the Mailchimp API Server
     *
     * This becomes true during constructing of this class if user entered a working API Key
     */
    private $is_Connected = false;




    /**
     * Initiate the Mailchimp API Object
     *
     * Fill the is_Connected and api property if successfully connected to the mailchimp server
     *
     */
    public function __construct()
    {

        require_once('library/mailchimp-api-v3.php');

        if ($this->get_api_key()) {
            $api = new WPDM_MailChimp_API_V3($this->get_api_key());
            $ping = $api->get('ping');

            if (isset($ping['health_status'])) {
                $this->is_Connected = true;
                $this->api = $api;
            }
        }
    }


    /**
     * get the Singleton Instance of the class
     *
     */
    static public function get_instance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    /**
     * Retrieves the wpdm mailchimp global settings options from the db
     *  @return array
     */
    public function get_wpdm_options()
    {
        return get_option(WPDM_MAILCHIMP_OPTION_KEY);
    }


    /**
     * Get the stored api key in the db
     *
     * @return string|false
     */
    public function get_api_key()
    {
        $options = $this->get_wpdm_options();
        $key_index = 'api_key';

        if (isset($options[$key_index]) && $options[$key_index]) {
            return sanitize_text_field($options[$key_index]);
        }

        return false;
    }


    /**
     * mailchimp connection status
     *
     * @return true|false
     */
    public function is_Connected()
    {
        return $this->is_Connected;
    }



    /**
     * @return array [['id', 'name']]
     *
     *
     */
    public function get_all_lists()
    {
        $items = [];
        if ($this->api) {
            $data = $this->api->get('lists');
            //echo '<pre>';print_r($list_data);echo '</pr>';
            if (isset($data['lists'])) {
                foreach ($data['lists'] as $item) {
                    $item = [
                        'id' => $item['id'],
                        'name' => $item['name']
                    ];
                    array_push($items, $item);
                }
            }
            update_option('__wpdm_mailchimp_lists', $items); // TODO do we need this??
        }

        return $items;
    }



    /**
     * get the data of a list
     */
    public function get_list($list_id)
    {
        if(empty($list_id)) return;

        if ($this->api) {
            $data = $this->api->get("lists/{$list_id}");
            //echo '<pre>';print_r($list_data);echo '</pr>';
            if (isset($data['id'])) {
                return $data;
            }
        }

        return false;
    }


    /**
     * get all interest-categories (AKA GROUP CATEGORIES) of a list
     *
     * @return array [[id,title]]
     */
    public function get_interest_categories($list_id)
    {
        if(empty($list_id)) return;

        $categories = [];
        if ($this->api) {
            $data = $this->api->get("lists/{$list_id}/interest-categories");
            if (isset($data['categories'])) {
                foreach ($data['categories'] as $category) {
                    array_push($categories, [
                        'id' => $category['id'],
                        'title' => $category['title']
                    ]);
                }
            }
        }
        return $categories;
    }


    /**
     * get interests (AKA groups) of a list && interest-category
     *
     * @return array [[id, title]] of interests
     */
    public function get_interests($list_id, $category_id)
    {
        if(empty($list_id) | empty($category_id)) return;

        $interests = [];
        if ($this->api) {
            $uri = "lists/{$list_id}/interest-categories/{$category_id}/interests";
            $data = $this->api->get($uri, ['count' => 1000]);
            //var_dump($uri);
            if (isset($data['interests'])) {
                foreach ($data['interests'] as $interest) {
                    array_push($interests, [
                        'id' => $interest['id'],
                        'name' => $interest['name']
                    ]);
                }
            }
        }

        return $interests;
    }



    /**
     * @param $list_id
     * @return array
     */
    public function get_merge_fields($list_id)
    {
        if(empty($list_id)) return [];

        $fields = [];
        if ($this->api) {

            $data = $this->api->get("lists/{$list_id}/merge-fields?count=1000");
            if (isset($data['merge_fields'])) {

                foreach ($data['merge_fields'] as $field) {
                    array_push($fields, $field);
                }
            }
        }
        return  $fields;
    }


    /**
     * Check if an email is exists in a mailchimp list
     *
     * @return true|false
     *
     */
    public function email_exists_in_list($email, $list_id)
    {
        if ($this->api) {
            $email_hash = $this->api->subscriberHash($email);
            $data = $this->api->get("lists/{$list_id}/members/{$email_hash}");

            if (is_array($data) && isset($data['id'])) return true;
        }
        return false;
    }


    // Adds an email to the mailchimp subscription list
    public function subscribeEmail($list_id, $email, $merge_fields, $interests, $email_type = 'html', $double_optin = false)
    {

        if ($this->api != null) {

            $result = $this->api->post(
                "lists/$list_id/members",
                [
                    'email_address' => $email,
                    'email_type'    => $email_type,
                    'status'        => $double_optin ? 'pending' : 'subscribed',
                    'merge_fields'  => $merge_fields,
                    'interests'     => $interests,
                ]
            );

            return $result;
        }
        return false;
    }
}
