<?php

namespace WPDM\WPPM\Access;

use Aws\Waf\WafClient;
use WPDM\__\UI;
use WPDM\WPPM\Plan\PlanCPT;
use WPDM\WPPM\Plan\PlanManager;
use WPDM\WPPM\Settings\SettingsManager;
use WPDM\WPPM\Subscription\SubscriptionManager;
use WPDM\WPPM\WPPM;

class AccessManager
{

    public static function getInstance()
    {
        static $instance;
        if (is_null($instance)) {
            $instance = new self;
        }
        return $instance;
    }


    /**
     *
     *
     */
    private function __construct()
    {
        $this->addHooks();
    }

    /**
     *
     *
     */
    public function addHooks()
    {
        add_action('template_include', array($this, 'sitewideProtection'));
        add_action('admin_init', array($this, 'addPostAccessMetabox'));
        add_action('save_post', array($this, 'savePostAccessRoles'));
        add_action('the_content', array($this, 'authorizeContentAccess'), 99);
        add_shortcode('wppm_access', array($this, 'wrappedContentAccess'));
        add_shortcode('wpmembership', array($this, 'wrappedContentAccess'));
    }

    function sitewideProtection($template)
    {
        if(!is_user_logged_in() && WPPM()->settings_manager->getGeneralSettingValue('lockdown')) {
            $template =  __DIR__.'/views/login.php';
        }
        return $template;
    }


    /**
     *
     *
     */
    public function addPostAccessMetabox()
    {
        add_meta_box('wppm_post_access', _x('Membership Access', 'metabox heading', WPPM_TEXTDOMAIN), array($this, 'renderPostAccessMetaBox'), 'page', 'side');
    }



    /**
     *
     *
     */
    public function authorizeContentAccess($content)
    {
        // Check if we're inside the main loop in a single Post.
        if (
            is_singular() &&
            in_the_loop() &&
            is_main_query() &&
            in_array(get_post_type(), SettingsManager::getInstance()->getGuardedPostTypes())
        ) {
            if ($this->canCurrentUserAccessPost(get_the_ID())) {
                return $content;
            } else {
                $redirect = "";

                $message = is_user_logged_in() ? WPPM()->settings_manager->getGeneralSettingValue('nonmembertext') : WPPM()->settings_manager->getGeneralSettingValue('notloggedintext');
                $footer = "";

                if(!is_user_logged_in()) {
                    $login_link = WPDM()->user->login->modalLoginFormBtn();
                    //$login_link = $login_link ?: "<a href='".WPPM()->account_manager->getAccountPagePermalink($_SERVER['REQUEST_URI'])."'>".esc_attr__( 'Login', WPPM_TEXTDOMAIN )."</a>";
                    if($login_link)
                        $footer = "<strong>".esc_attr__('Already Subscribed?', WPPM_TEXTDOMAIN) . " " . $login_link."</strong>";
                }
                $message = str_replace("!!levels!!", $this->getAccessiblePlanLinks(get_the_ID(), "btn btn-danger btn-sm pl-2 pr-2 pt-0 pb-0", " or "), $message);
                $message = [$message];
                $url = WPPM()->account_manager->getAccountPagePermalink($_SERVER['REQUEST_URI']);
                if((int)WPPM()->settings_manager->getGeneralSettingValue("redirect") === 1) $redirect = UI::div("Redirecting...<script>location.href='{$url}'</script>", "alert alert-info lead text-center", ['style' => 'width: 400px;margin: 15px auto;']);
                $login_form = $footer || $redirect ? "" : UI::div(WPDM()->user->login->form(), "", ['style' => 'width: 400px;margin: 20px auto;']);
                return UI::div(UI::card(esc_attr__( 'Restricted Content', WPPM_TEXTDOMAIN ), $message, $footer, ['class' => 'text-center', 'style' => 'width: 400px;margin: 0 auto;line-height: 1.8;']), "w3eden").$redirect.$login_form;
            }
        }
        $content = apply_filters("wppm_restricted_content_message", $content, get_the_ID());
        return $content;
    }



    /**
     *
     *
     *
     */
    public function renderPostAccessMetaBox()
    {
        global $post;
        $plans = PlanManager::getInstance()->getPlans();
        $post_role_ids = $this->getPostRoleIds($post->ID);
?>
        <ul id="membershipschecklist" class="list:category categorychecklist form-no-clear">
            <input type="hidden" name="wppm_post_access_nonce" value="<?php echo wp_create_nonce(plugin_basename(__FILE__)) ?>" />
            <?php foreach ($plans as $plan) : ?>
                <li>
                    <label class="selectit">
                        <input type="checkbox" <?= in_array($plan->getRoleId(), $post_role_ids) ? "checked" : "" ?> name="__wpdm_access[]" value="<?= $plan->getRoleId() ?>" />
                        <?= $plan->getName(); ?>
                    </label>
                </li>
            <?php endforeach ?>
        </ul>
<?php
    }


    /**
     *
     *
     */
    public function savePostAccessRoles($post_id)
    {
        $is_valid_nonce = false;
        if (isset($_POST['wppm_post_access_nonce']) && wp_verify_nonce($_POST['wppm_post_access_nonce'], plugin_basename(__FILE__))) {
            $is_valid_nonce = true;
        }

        $is_autosave = wp_is_post_autosave($post_id);
        $is_revision = wp_is_post_revision($post_id);

        if (
            $is_autosave ||
            $is_revision ||
            !$is_valid_nonce ||
            !current_user_can('edit_post', $post_id) ||
            get_post_type($post_id) === 'wpdmpro'
        )
            return;


        $selected_roles = $_POST[WPPM_POST_ACCESS_ROLES_KEY] ?: [];
        update_post_meta($post_id, WPPM_POST_ACCESS_ROLES_KEY, $selected_roles);
    }


    /**
     * [wppm_1, wppm_2, ...]
     *
     * @return string[]
     */
    public function getPostRoleIds($post_id)
    {
        $role_ids = maybe_unserialize(get_post_meta($post_id, WPPM_POST_ACCESS_ROLES_KEY, true));
        return is_array($role_ids) ? $role_ids : [];
    }


    /**
     *
     * @return WP_Term[]
     */
    public function getPostTerms($post_id)
    {
        $taxonomies = PlanCPT::getInstance()->getAttachableTaxonomyNames();
        return wp_get_post_terms($post_id, $taxonomies);
    }


    /**
     * Required Plan(s) for a post
     *
     * algorithm :
     *  get the required roles related plans for the post if any
     *  get the post terms related plans if any
     *
     * @return Plan[]
     */
    public function getRequiredPlans($post_id)
    {
        $post_role_ids = $this->getPostRoleIds($post_id);
        $post_terms = $this->getPostTerms($post_id);
        $terms_related_plan_ids = $this->getTermsRelatedPlanIds($post_terms);

        $all_plans = PlanManager::getInstance()->getPlans();
        $required_plans = [];
        foreach ($all_plans as $p) {
            if (
                in_array($p->getRoleId(), $post_role_ids) ||
                in_array($p->getId(), $terms_related_plan_ids)
            ) {
                array_push($required_plans, $p);
            }
        }

        return $required_plans;
    }


    /**
     * Algorithm
     *  true for admin
     *  get required plans of the post
     *  get active plans of the user
     *  if if any common plan found then true
     *
     *
     * @return boolean
     */
    public function canUserAccessPost($user_id, $post_id)
    {
        //no restriction for admin
        //if (current_user_can('manage_options')) return true;

        //$required_plans = $this->getRequiredPlans($post_id);
        //$required_plan_ids = wp_list_pluck($required_plans, 'id');
        $required_plan_ids = $this->getPostRoleIds($post_id);

        if (!count($required_plan_ids)) return true;

        //$user_active_plan_ids = SubscriptionManager::getInstance()->getActivePlanIdsOfUser($user_id);
        $user_active_plan_ids = WPPM()->subscription_manager->getActivePlans($user_id);
        //foreach($required_plan_ids as &$required_plan_id){ $required_plan_id = "wppm_".$required_plan_id; }

        $common_plan_ids = array_intersect($required_plan_ids, $user_active_plan_ids);

        return count($common_plan_ids) > 0;
    }


    /**
     * Get the plan ids associated with the given terms
     *
     * @return int[]
     */
    public function getTermsRelatedPlanIds($terms = [])
    {
        if (count($terms) <= 0) return [];

        $tax_queries = [
            'relation' => 'OR'
        ];

        foreach ($terms as $t) {
            $tax_queries[] = [
                'taxonomy' => $t->taxonomy,
                'field'    => 'term_id',
                'terms'    => $t->term_id
            ];
        }

        $args = array(
            'post_type' => 'wppm_plan',
            'tax_query' => $tax_queries
        );
        $plan_posts = get_posts($args);
        return wp_list_pluck($plan_posts, 'ID');
    }


    /**
     *
     * @return boolean
     */
    public function canCurrentUserAccessPost($post_id)
    {
        return $this->canUserAccessPost(get_current_user_id(), $post_id);
    }



    /**
     *
     * @return string
     */
    public function getAccessiblePlanLinks($post_id, $btn_class = '', $sap = " / ")
    {
        $anchors = [];
        foreach ($this->getRequiredPlans($post_id) as $p) {
            $anchors[] = "<a class='{$btn_class}' href='{$p->getCheckoutUrl()}'> {$p->getName()}</a>";
        }
        return implode($sap, $anchors);
    }


    /**
     * shortcode [wppm_access plans='pro plan']
     *
     */
    public function wrappedContentAccess($atts, $content = null)
    {
        $has_access = false;
        //admin has access by default
        if(current_user_can('manage_options')) $has_access = true;

        $required_plan_names = wpdm_valueof($atts, 'plans');
        $required_plan_names = explode(',', $required_plan_names);
        //sanitize the names to slug for better comparison
        array_walk($required_plan_names, function(&$v, &$k){
            $v = sanitize_title($v); //Pro Plan ====> pro-plan
        });

        $user_plans = SubscriptionManager::getInstance()->getActivePlansOfUser(get_current_user_id());
        foreach($user_plans as $p) {
            $plan_name = sanitize_title($p->getName());
            if(in_array($plan_name, $required_plan_names)){
                $has_access = true;
            }
        }
        if($has_access){
            return do_shortcode($content);
        }else{
            return "";
        }
    }
}
