<?php

namespace WPDM\WPPM\Account;

use WPDM\WPPM\Plan\Plan;
use WPDM\WPPM\Subscription\Subscription;

if (!defined('ABSPATH')) exit;


class AccountManager
{
    /**
     * holds singleton instance of the class
     */
    private static $instance;


    /**
     * private constructor of the class
     *
     */
    private function __construct()
    {
        $this->loadDependencies();
        $this->addHooks();
    }


    /**
     * get the singleton instance of the class
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    /**
     * Loads dependency files
     *
     */
    public function loadDependencies()
    {
    }

    /**
     * add hooks
     */
    public function addHooks()
    {
        add_shortcode("wppm_account", array($this, "myAccount"));
        add_filter('wpdm_dashboard_menu', [$this, 'dashboardMenuItems'], 10, 1);
    }

    /**
     * @param $menu_items
     * @return mixed
     */
    function dashboardMenuItems($menus)
    {
        $account = $menus['account'];
        unset($menus['account']);
        $menus['promembership'] = [
            'title' => __('Membership', WPPM_TEXTDOMAIN),
            'items' => [
                'membership-plans' => ['name' => __('Subscriptions', 'text-domain'), 'shortcode' => '[wppm_account]', 'icon' => 'fa fa-crown color-info mr-3'],
                'membership-items' => ['name' => __('My Downloads', 'text-domain'), 'callback' => [$this, 'myDownloads'], 'icon' => 'far fa-arrow-alt-circle-down color-info mr-3']
            ]
        ];
        $menus['account'] = $account;
        return $menus;
    }

    /**
     * Show user's downloads according to their subscriptions
     * @return false|string
     */
    function myDownloads()
    {
        ob_start();
        include __DIR__ . '/user-downloads.php';
        return ob_get_clean();
    }

    /**
     * User account
     *
     */
    public function myAccount($params = [])
    {
        if (!is_user_logged_in()) return WPDM()->user->login->form($params);

        $subscriptions = WPPM()->subscription_manager->getActiveSubscriptionsOfCurrentUser();

        $args = [
            'subscriptions' => $subscriptions
        ];

        $file = __DIR__ . '/my-account.php';
        return wppm_render_template($file, $args, false);
    }


    /**
     *
     *
     */
    public function renderUserAccount()
    {

        $user_id = wpdm_query_var('user_id');
        $user = get_user_by('id', $user_id);
        if (!$user) {
            echo '<p>No User Selected</p>';
            exit;
        }

        if (isset($_REQUEST[WPPM_USER_PLAN_SELECTION_NONCE]) && wp_verify_nonce($_REQUEST[WPPM_USER_PLAN_SELECTION_NONCE])) {
            $user_subs = wpdm_query_var(WPPM_USER_SUBSCRIPTIONS) ?: [];
            //pd($user_subs);
            $this->updateUserSubscriptions($user_id, $user_subs);
        }


        $subscriptions = WPPM()->subscription_manager->getSubscriptionsOfUser($user_id);

        $args = [
            'user' => $user,
            'subscriptions' => $subscriptions
        ];

        $file = __DIR__ . '/user-account.php';
        return wppm_render_template($file, $args, true);
    }


    /**
     * Get the url of wppm account page
     *
     * @return string|false
     */
    public function getAccountPagePermalink($redirect = '')
    {
        $account_page_id = (int)WPPM()->settings_manager->getGeneralSettingValue('wppm_account_page_id');
        $wpdm_user_dboard = (int)get_option('__wpdm_user_dashboard');
        $url = get_permalink($account_page_id);
        if($redirect !== '') $url = add_query_arg(['redirect_to' => $redirect], $url);
        if($account_page_id === $wpdm_user_dboard)
            $url = add_query_arg(['udb_page' => 'membership-plans'], $url);
        return $url;
    }


    /**
     *
     * @return Redirection
     */
    public function redirectToAccountPage()
    {
        $redirect_url = $this->getAccountPagePermalink() ?: get_home_url();
        wp_safe_redirect($redirect_url);
        exit;
    }


    /**
     * add wppm role selction
     *
     */
    function renderUserPlansSelection($user_id)
    {
        $subscribable_plans = WPPM()->plan_manager->getSubscribablePlans();
        $user_active_subscriptions = WPPM()->subscription_manager->getActiveSubscriptionsOfUser($user_id);
        $user_active_plan_ids = WPPM()->subscription_manager->getActivePlanIdsOfUser($user_id);

        //plan_id, plan_name, has_subscribed, subscription_id, current_period_end,
        $plan_list = [];

        foreach ($subscribable_plans as $plan) {
            $sub = $this->getSubscriptionByPlanId($user_active_subscriptions, $plan->getId());

            $p = [
                'plan_id' => $plan->getId(),
                'plan_name' => $plan->getName(),
                'subscribed' => in_array($plan->getId(), $user_active_plan_ids),
                'subscription_id' => $sub ? $sub->getId() : null,
                'expires_at' => $sub && $sub->getExpiresAt() ? $sub->getExpiresAt()->format('Y-m-d') : ""
            ];
            array_push($plan_list, $p);
        }

        // pd($plan_list);

        $args = [
            'plans' => $plan_list,
        ];

        //pd($user->roles);

        $file = plugin_dir_path(__FILE__) . 'user-plans-selection.php';
        wppm_render_template($file, $args);
    }


    private function getSubscriptionByPlanId($subs, $plan_id)
    {
        $filtered_subs = array_filter($subs, function ($s) use ($plan_id) {
            return $s->getPlanId() == $plan_id;
        });
        $sub = array_shift($filtered_subs);

        return $sub;
    }


    /**
     * Update user subscriptions manually
     *
     */
    public function updateUserSubscriptions($user_id, $user_subs)
    {
        $user = get_user_by('id', $user_id);
        $new_subs = [];
        $cancel_subs = [];
        $update_subs = [];
		//wpdmdd($user_subs);
        foreach ($user_subs as $s) {
            if (empty($s['subscription_id'])) {
                //NEW SUBS
                if (isset($s['subscribed']) && $s['subscribed'] == 1) {
                    $subscription = new Subscription();
                    $subscription->setUserId($user_id)
                        ->setPlanId($s['plan_id'])
                        ->setPaymentSystemId('manual')
                        ->setExpiresAt($s['expires_at'])
                        ->setSubscriptionStatusActive();

                    $subscription->save();
                    array_push($new_subs, $subscription);
                } else {
					$plan = Plan::get($s['plan_id'])->getRoleId();
					$user->remove_role($plan);
                }
            } else {
                //EXISTING SUBS
                //Cancel Subs
                if (!isset($s['subscribed'])) {
                    $subscription = Subscription::get($s['subscription_id']);
                    $subscription->cancel();
                    array_push($cancel_subs, $subscription);
                }
                //update subs
                elseif ($s['expires_at'] != $s['expires_at_old']) {
                    $subscription = Subscription::get($s['subscription_id']);
                    $subscription->setExpiresAt($s['expires_at']);
                    $subscription->save();
                    array_push($update_subs, $subscription);
                }
            }
        }

        if (count($new_subs) > 0 || count($cancel_subs) > 0 || count($update_subs)) {
            do_action('wppm_user_subscriptions_changed', $user, [
                'new' => $new_subs,
                'canceled' => $cancel_subs,
                'updated' => $update_subs
            ]);
        }
    }

}
