<?php

namespace WPDM\WPPM\Core;

use Exception;
use WPDM\WPPM\Plan\PlanCPT;
use WPDM\WPPM\Subscription\Subscription;

if (!defined('ABSPATH')) exit;

class InstallManager
{
    /**
     * holds singleton instance of the class
     */
    private static $instance;

    /**
     * private constructor of the class
     * so this class cannot be created outside
     *
     */
    private function __construct()
    {
        $this->addHooks();
    }


    /**
     * get the singleton instance of the class
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }

        return self::$instance;
    }


    /**
     *
     * hooks
     */
    public function addHooks()
    {
        if (is_admin()) {
            add_action('admin_notices', array($this, 'pluginDependencyNotice'));
        }
    }



    public function activate()
    {
        //Setup data tables
        $this->install();
        $this->migrateV1();
    }



    public function deactivate()
    {
    }


    /**
     *
     * for test purpose
     */
    public function renderDataMigration()
    {
        $this->migrateV1();
    }




    public function install()
    {

        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();

        $subscriptions_table_name = $wpdb->prefix . "wppm_subscriptions";

        $sqls[] = "CREATE TABLE IF NOT EXISTS $subscriptions_table_name (
            `id` INT NOT NULL AUTO_INCREMENT,
            `user_id` INT NOT NULL,
            `plan_id` INT NOT NULL,
            `subscription_status` VARCHAR(50) DEFAULT 'INCOMPLETE' NOT NULL,
            `expires_at` timestamp NULL,
            `expiration_notified_at` timestamp NULL,

            `payment_system_id` VARCHAR(50) NOT NULL,
            `payment_system_subscription_id` VARCHAR(50) NULL,
            `payment_system_subscription_data` text NULL,

            `created_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
            `modified_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

            PRIMARY KEY  (id)
        ) $charset_collate;";

        //invoices table helps to cache the invoices from payment system.
        // in paypal invoice is similar to orders
        $sqls[] = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}wppm_invoices` (
          `id` int(11) NOT NULL AUTO_INCREMENT,
          `subscription_id` int(11) NOT NULL,
          `amount` DOUBLE NULL,
          `currency_code` VARCHAR(10) NULL,

          `payment_system_id` VARCHAR(50) NOT NULL,
          `payment_system_invoice_id` VARCHAR(80) NULL,
          `payment_system_invoice_data` text NULL,

          `created_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
          `modified_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
          PRIMARY KEY (`id`)
        )";

        $sqls[] = "ALTER TABLE {$subscriptions_table_name} ADD `expiration_notified_at` timestamp NULL";

        foreach ($sqls as $sql) {
            $wpdb->query($sql);
        }

        $this->createCouponsTable();
    }


    /**
     *
     *
     */
    public function createCouponsTable()
    {
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();

        $table_name = $wpdb->prefix . "wppm_coupons";

        $sql = "CREATE TABLE IF NOT EXISTS $table_name (
            `id` INT NOT NULL AUTO_INCREMENT,
            `code` VARCHAR(50) NOT NULL,
            `description` TEXT NULL,
            `discount_type` VARCHAR(50) NOT NULL,
            `discount_amount` DOUBLE NOT NULL,
            `plan_ids` TEXT NULL,
            `usage_limit` INT DEFAULT 0,
            `usage_count` INT DEFAULT 0,
            `expires_at` timestamp NULL,

            `created_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
            `modified_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,

            PRIMARY KEY  (id)
        ) $charset_collate;";

        $wpdb->query($sql);
    }



    /**
     * data migration
     *
     */
    public function migrateV1()
    {
        $migration = get_option("wppm_data_migration");
        if ($migration === "v1") return;


        global $wpdb;
        $old_plans_table = $wpdb->prefix . "wpmp_membership_levels";
        $old_subscriptions_table = $wpdb->prefix . "wpmp_memberships_users";
        $old_invoices_table = $wpdb->prefix . "wpmp_membership_orders";

        $new_subscriptions_table =  $wpdb->prefix . "wppm_subscriptions";
        $new_invoices_table =  $wpdb->prefix . "wppm_invoices";

        $old_plans = $wpdb->get_results("SELECT * FROM $old_plans_table", ARRAY_A);

        if (!$old_plans) return;

        foreach ($old_plans as $old_plan) {
            $metas = [
                'role_id' => 'wppm_' . $old_plan['id'],
                'confirmation' => $old_plan['confirmation'],
                'features' => $old_plan['features'],
                'initial_amount' => $old_plan['initial_payment'],
                'billing_amount' => $old_plan['billing_amount'],
                'billing_limit' => $old_plan['billing_limit'],
                'billing_period_count' => $old_plan['cycle_number'],
                'billing_period' => $old_plan['cycle_period'] ? strtoupper($old_plan['cycle_period']) : null,
                'trial_amount' => $old_plan['trial_amount'],
                'trial_period_count' =>  $old_plan['trial_limit'],
                'allow_subscription' => $old_plan['allow_signups'] == 1 ? 'YES' : 'NO'
            ];

            $post_array = [
                'post_content' => $old_plan['description'],
                'post_title' => $old_plan['name'],
                'post_type' => 'wppm_plan',
                'post_status' => 'publish',
            ];

            PlanCPT::getInstance()->actionRegisterPostType();

            $post_id =  wp_insert_post($post_array, true);

            foreach ($metas as $key => $value) {
                update_post_meta(
                    $post_id,
                    WPPM_PLAN_META_PREFIX . $key,
                    $value
                );
            }

            $old_invoices = $wpdb->get_results("SELECT * FROM $old_invoices_table WHERE `membership_id` = '{$old_plan['id']}' ", ARRAY_A);
            if (empty($old_invoices)) continue;

            foreach ($old_invoices as $old_invoice) {
                $old_subscriptions = $wpdb->get_results("SELECT * FROM $old_subscriptions_table WHERE `membership_id` = '{$old_plan['id']}' AND `user_id` = '{$old_invoice['user_id']}' ", ARRAY_A);

                if (empty($old_subscriptions)) continue;

                $old_sub = current($old_subscriptions);

                $payment_system_subscription_id = $old_invoice['code'];

                $payment_system_id = $old_invoice['payment_type'];
                //if payment system sub id starts with I- then it is paypal //else keep it as it is
                if (substr($payment_system_subscription_id, 0, 2) == 'I-') {
                    $payment_system_id = 'paypal';
                }

                $payment_system_id = strtolower($payment_system_id);

                $new_subscription = Subscription::getByPaymentSystemSubscriptionId($payment_system_subscription_id);

                if (!$new_subscription) {
                    $new_subscription_data = [
                        'plan_id' => $post_id,
                        'user_id' => $old_sub['user_id'],
                        'expires_at' => $old_sub['enddate'],
                        'expiration_notified_at' => null,
                        'subscription_status' => $old_sub['status'] ? strtoupper($old_sub['status']) : null,
                        'payment_system_id' => $payment_system_id,
                        'payment_system_subscription_id' => $payment_system_subscription_id,
                        'payment_system_subscription_data' => NULL,
                        'created_at' => $old_sub['startdate']
                    ];

                    $wpdb->insert(
                        $new_subscriptions_table,
                        $new_subscription_data
                    );
                    $sub_id = $wpdb->insert_id;
                } else {
                    $sub_id = $new_subscription->getId();
                }


                $new_invoice_data = [
                    'subscription_id' => $sub_id,
                    'amount' => $old_invoice['subtotal'],
                    'currency_code' => WPPM()->currency_code,
                    'payment_system_id' =>  $payment_system_id,
                    'payment_system_invoice_id' => $old_invoice['code'], //TODO
                    'payment_system_invoice_data' => NULL,
                    'created_at' => $old_invoice['timestamp']
                ];

                $wpdb->insert(
                    $new_invoices_table,
                    $new_invoice_data
                );
            }
        }

        update_option("wppm_data_migration", "v1");
    }






    public function pluginDependencyNotice()
    {
        if (!function_exists('wpdmpp_currency_sign')) {   ?>
            <div id="wpdm-activate-shop">
                <div class="updated error" id="wpdm-activate-shop-info" style="padding: 20px;">
                    <?php _e("Required plugin is missing.", "wppromembership"); ?> <a style="font-weight: 800;color:#d34452" href="#" id="wpdm-activate-shop-link"><?php _e('Click here to install and active', 'wppromembership');   ?></a>
                </div>
                <script>
                    jQuery(function($) {
                        $('#wpdm-activate-shop-link').on('click', function() {
                            $(this).html('Activating...');
                            $.post(ajaxurl, {
                                action: 'wpdm-activate-shop'
                            }, function(res) {
                                $('#wpdm-activate-shop-info').html(res);
                            });
                            return false;
                        });
                    });
                </script>
            </div>
<?php
            return false;
        }
        return true;
    }
}
