<?php

namespace WPDM\WPPM\Coupon;

use DateTime;
use Exception;
use WPDM\WPPM\Plan\Plan;

class Coupon
{
    /**
     * @var int|null
     */
    private $id;

    /**
     * @var string
     */
    private $code;

    /**
     * @var string|null
     */
    private $description;

    /**
     * @var string PERCENT | FIXED
     */
    private $discount_type;

    /**
     * @var float 
     */
    private $discount_amount;

    /**
     * @var int[]|null
     */
    private $plan_ids;

    /**
     * @var int
     */
    private $usage_limit;

    /**
     * @var int
     */
    private $usage_count;

    /**
     * @var DateTime|null
     */
    private $expires_at;

    /**
     * @var DateTime|null
     */
    private $created_at;

    /**
     * @var DateTime|null
     */
    private $modified_at;



    public function __construct($data = null)
    {
        if (is_array($data)) {
            $this->fromArray($data);
        }
    }


    public function fromArray($data = [])
    {
        $this->setId(wpdm_valueof($data, 'id'));
        $this->setCode(wpdm_valueof($data, 'code'));
        $this->setDescription(wpdm_valueof($data, 'description'));
        $this->setDiscountType(wpdm_valueof($data, 'discount_type'));
        $this->setDiscountAmount(wpdm_valueof($data, 'discount_amount'));
        $this->setPlanIds(maybe_json_decode(wpdm_valueof($data, 'plan_ids'))); //
        $this->setUsageLimit(wpdm_valueof($data, 'usage_limit'));
        $this->setUsageCount(wpdm_valueof($data, 'usage_count'));
        $this->setExpiresAt(wpdm_valueof($data, 'expires_at'));
        $this->setCreatedAt(wpdm_valueof($data, 'created_at'));
        $this->setModifiedAt(wpdm_valueof($data, 'modified_at'));
    }


    /**
     * 
     * @return string
     */
    public static function getTable()
    {
        global $wpdb;
        return $wpdb->prefix . 'wppm_coupons';
    }


    /**
     * Save (create or update) the obj in db
     * 
     * @return int|Exception
     */
    public function save()
    {
        global $wpdb;
        $coupon_table = self::getTable();

        $values = [
            'code' => $this->getCode(),
            'description' => $this->getDescription(),
            'discount_type' => $this->getDiscountType(),
            'discount_amount' => $this->getDiscountAmount(),
            'plan_ids' => maybe_json_encode($this->getPlanIds()),
            'usage_limit' => $this->getUsageLimit(),
            'usage_count' => $this->getUsageCount(),
            'expires_at' => $this->getExpiresAt() ? $this->getExpiresAt()->format('Y-m-d H:i:s') : null
        ];


        if ($this->getId()) {
            //update
            $result = $wpdb->update(
                $coupon_table,
                $values,
                ['id' => $this->getId()]
            );
        } else {
            //create
            $result = $wpdb->insert(
                $coupon_table,
                $values
            );

            if ($result) {
                $this->id = $wpdb->insert_id;
            }
        }

        if ($result === false) {
            $msg = $wpdb->last_error . ' from ' . $wpdb->last_query;
            throw new Exception($msg);
        }

        return $this->getId();
    }


    /**
     * 
     * @return int|false
     */
    public function delete()
    {
        global $wpdb;

        return $wpdb->delete(self::getTable(), ['id' => $this->getId()]);
    }


    /**
     * Static method
     * 
     * 
     * @return Coupon|null
     * 
     */
    public static function get($coupon_id)
    {
        global $wpdb;
        $table = self::getTable();

        $rows = $wpdb->get_results("SELECT * FROM $table WHERE `id` = {$coupon_id}", ARRAY_A);

        if (count($rows)) {
            $row = array_shift($rows);
            return new Coupon($row);
        }

        return null;
    }


    /**
     * 
     * @return Coupon[]
     */
    public static function getAll()
    {
        global $wpdb;
        $table = self::getTable();

        $rows = $wpdb->get_results("SELECT * FROM $table order by `created_at` DESC ", ARRAY_A);

        $coupons = [];
        if (count($rows)) {
            foreach ($rows as $row) {
                array_push($coupons,  new Coupon($row));
            }
        }

        return $coupons;
    }


    /**
     * Get by coupon code 
     * 
     * @param string $code
     * 
     * @return self|null
     */
    public static function getByCode(string $code)
    {
        global $wpdb;
        $table = self::getTable();
        $code = strtoupper($code);
        $rows = $wpdb->get_results("SELECT * FROM $table WHERE `code` = '{$code}'", ARRAY_A);

        if (count($rows)) {
            $row = array_shift($rows);
            return new Coupon($row);
        }

        return null;
    }

    /**
     * Increase usage count by one
     * 
     */
    public function increaseUsageCount()
    {
        $this->setUsageCount($this->getUsageCount() + 1);
        $this->save();
        return $this;
    }


    /**
     * coupon is only valid if 1. usage_count is less than usage_limit AND 2. current time is less than expires_at
     * 
     * @return boolean
     */
    public function isValid()
    {
        if (
            ($this->getUsageLimit() &&  ($this->getUsageLimit() <= $this->getUsageCount())) ||
            ($this->getExpiresAt() && ($this->getExpiresAt() < new DateTime()))
        ) {
            return false;
        }

        return true;
    }


    /**
     * 
     * @return boolean
     */
    public function isApplicableInPlan(int $plan_id)
    {

        if (
            !$this->isValid() ||
            ($this->getPlanIds() && !in_array($plan_id, $this->getPlanIds()))
        ) {
            return false;
        }

        return true;
    }

    /**
     * 
     * @return boolean
     */
    public function isPercentDiscount()
    {
        return $this->getDiscountType() == 'PERCENT';
    }


    /**
     * 
     * @return boolean
     */
    public function isFixedDiscount()
    {
        return $this->getDiscountType() == 'FIXED';
    }


    /**
     * 
     * @return string
     */
    public function getDiscountText()
    {
        if ($this->isPercentDiscount()) return $this->getDiscountAmount() . ' %';
        if ($this->isFixedDiscount()) return ' $ ' . $this->getDiscountAmount();
    }


    /**
     * 
     * @return string
     */
    public function getUsageText()
    {
        $text =  (string) $this->getUsageCount() . " / ";
        $text .=  $this->getUsageLimit() != 0 ? (string) $this->getUsageLimit() : "&#8734";
        return  $text;
    }


    public function getApplicablePlanNames()
    {
        $plan_names = [];
        foreach ($this->getPlanIds() as $plan_id) {
            $plan = Plan::get($plan_id);
            if ($plan) {
                $name = $plan->getName();
                $name = strtoupper($name);
                $name = "<span class='badge badge-secondary'>{$name}</span>";
                array_push($plan_names, $name);
            }
        }

        return implode(" ", $plan_names) ? : 'ALL PLANS';
    }


    /**
     * Get the value of id
     *
     * @return  int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set the value of id
     *
     * @param  int  $id
     *
     * @return  self
     */
    public function setId($id)
    {
        $this->id = $id;

        return $this;
    }

    /**
     * Get the value of code
     *
     * @return  string
     */
    public function getCode()
    {
        return strtoupper($this->code);
    }

    /**
     * Set the value of code
     *
     * @param  string  $code
     *
     * @return  self
     */
    public function setCode(string $code)
    {
        $this->code = $code;

        return $this;
    }

    /**
     * Get pERCENTAGE | FIXED
     *
     * @return  string
     */
    public function getDiscountType()
    {
        return $this->discount_type;
    }

    /**
     * Set pERCENTAGE | FIXED
     *
     * @param  string  $discount_type  PERCENTAGE | FIXED
     *
     * @return  self
     */
    public function setDiscountType(string $discount_type)
    {
        $this->discount_type = $discount_type;

        return $this;
    }

    /**
     * Get the value of discount_amount
     *
     * @return  float
     */
    public function getDiscountAmount()
    {
        return number_format($this->discount_amount, 2);
    }

    /**
     * Set the value of discount_amount
     *
     * @param  float  $discount_amount
     *
     * @return  self
     */
    public function setDiscountAmount(float $discount_amount)
    {
        $this->discount_amount = $discount_amount;

        return $this;
    }

    /**
     * Get the value of plan_id
     *
     * @return  int[]
     */
    public function getPlanIds()
    {
        return maybe_json_decode($this->plan_ids) ?: [];
    }

    /**
     * Set the value of plan_id
     *
     * @param  int[]|null  $plan_id
     *
     * @return  self
     */
    public function setPlanIds($plan_ids)
    {
        $this->plan_ids = $plan_ids;

        return $this;
    }

    /**
     * Get the value of usage_limit
     *
     * @return  int
     */
    public function getUsageLimit()
    {
        return (int) $this->usage_limit;
    }

    /**
     * Set the value of usage_limit
     *
     * @param  int  $usage_limit
     *
     * @return  self
     */
    public function setUsageLimit($usage_limit)
    {
        $this->usage_limit = $usage_limit;

        return $this;
    }

    /**
     * Get the value of usage_count
     *
     * @return  int
     */
    public function getUsageCount()
    {
        return (int) $this->usage_count;
    }

    /**
     * Set the value of usage_count
     *
     * @param  int  $usage_count
     *
     * @return  self
     */
    public function setUsageCount($usage_count)
    {
        $this->usage_count = $usage_count;

        return $this;
    }

    /**
     * Get the value of expires_at
     *
     * @return  DateTime|null
     */
    public function getExpiresAt()
    {
        return wppm_get_date_time($this->expires_at);
    }

    /**
     * Set the value of expires_at
     *
     * @param  DateTime|null  $expires_at
     *
     * @return  self
     */
    public function setExpiresAt($expires_at)
    {
        $this->expires_at = $expires_at;

        return $this;
    }

    /**
     * Get the value of created_at
     *
     * @return  DateTime|null
     */
    public function getCreatedAt()
    {
        return  wppm_get_date_time($this->created_at);
    }

    /**
     * Set the value of created_at
     *
     * @param  DateTime|null  $created_at
     *
     * @return  self
     */
    public function setCreatedAt($created_at)
    {
        $this->created_at = $created_at;

        return $this;
    }

    /**
     * Get the value of modified_at
     *
     * @return  DateTime|null
     */
    public function getModifiedAt()
    {
        return wppm_get_date_time($this->modified_at);
    }

    /**
     * Set the value of modified_at
     *
     * @param  DateTime|null  $modified_at
     *
     * @return  self
     */
    public function setModifiedAt($modified_at)
    {
        $this->modified_at = $modified_at;

        return $this;
    }


    /**
     * Get the value of description
     *
     * @return  string|null
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Set the value of description
     *
     * @param  string|null  $description
     *
     * @return  self
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }
}
