<?php

namespace WPDM\WPPM\Invoice;

use Exception;

if (!defined('ABSPATH')) exit;



class Invoice
{
    /**
     * wppm subscription id
     */
    private $id;

    /**
     * wppm subscription id
     */
    private $subscription_id;

    /**
     * amount
     */
    private $amount;

    /**
     * Currency code
     */
    private $currency_code;

    /**
     * payment system id
     */
    private $payment_system_id;

    /**
     * payment system invoice id
     */
    private $payment_system_invoice_id;

    /**
     * JSON encoded payment system invoice data
     */
    private $payment_system_invoice_data;

    /**
     * created timestamp
     */
    private $created_at;

    /**
     * Invoice parser
     */
    private $_invoice_parser;



    /**
     * constructor
     *
     * this object can be created by passing data array
     *
     */
    public function __construct($data = null)
    {
        switch (gettype($data)) {
            case 'NULL':
                break;
            case 'array':
                $this->fromArray($data);
                break;
            default:
        }
    }

    /**
     *
     * @return void
     */
    private function fromArray($data = [])
    {
        $this->setId(wpdm_valueof($data, 'id'));
        $this->setSubscriptionId(wpdm_valueof($data, 'subscription_id'));
        $this->setAmount(wpdm_valueof($data, 'amount'));
        $this->setCurrencyCode(wpdm_valueof($data, 'currency_code'));

        $this->setPaymentSystemId(wpdm_valueof($data, 'payment_system_id'));
        $this->setPaymentSystemInvoiceId(wpdm_valueof($data, 'payment_system_invoice_id'));
        $this->setPaymentSystemInvoiceData(wpdm_valueof($data, 'payment_system_invoice_data'));

        $this->setCreatedAt(wpdm_valueof($data, 'created_at'));
    }


    /**
     *
     * @return InvoiceParser
     */
    public function getParser()
    {
        if(!$this->getPaymentSystem())
            return  false;

        if (empty($this->_invoice_parser)) {
            $paymentSystem = $this->getPaymentSystem();
            if(is_object($paymentSystem) && method_exists($paymentSystem, 'getInvoiceParser'))
                $this->_invoice_parser = $paymentSystem->getInvoiceParser();
        }
        if($this->_invoice_parser)
            $this->_invoice_parser->setData($this->getPaymentSystemInvoiceData());
        return $this->_invoice_parser;
    }


    /**
     *
     * @return Payment_System|null
     */
    public function getPaymentSystem()
    {
        return WPPM()->payment_system_manager->getPaymentSystem($this->getPaymentSystemId());
    }



    /**
     * Save (create or update) the obj in db
     *
     * @return id|Error
     */
    public function save()
    {
        global $wpdb;
        $invoice_table = $wpdb->prefix . 'wppm_invoices';

        $values = [
            'subscription_id' => $this->getSubscriptionId(),
            'amount' => $this->getAmount(),
            'currency_code' => $this->getCurrencyCode(),
            'payment_system_id' => $this->getPaymentSystemId(),
            'payment_system_invoice_id' => $this->getPaymentSystemInvoiceId(),
            'payment_system_invoice_data' => maybe_json_encode($this->getPaymentSystemInvoiceData())
        ];

        $formats = [
            '%d',
            '%f',
            '%s',
            '%s',
            '%s',
            '%s'
        ];

        if ($this->getId()) {
            //update
            $result = $wpdb->update(
                $invoice_table,
                $values,
                ['id' => $this->getId()],
                $formats,
                ['%d']
            );
        } else {
            //create
            $result = $wpdb->insert(
                $invoice_table,
                $values,
                $formats
            );

            if ($result) {
                $this->id = $wpdb->insert_id;
            }
        }

        if ($result === false) {
            $msg = $wpdb->last_error . ' from ' . $wpdb->last_query;
            throw new Exception($msg);
        }

        return $this->getId();
    }


    /**
     * Static method
     *
     *
     * @return array [Invoice]
     */
    public static function getBySubscriptionId($sub_id)
    {
        global $wpdb;
        $invoices = [];

        $invoices_table = $wpdb->prefix . 'wppm_invoices';

        $rows = $wpdb->get_results("SELECT * FROM $invoices_table WHERE `subscription_id` = {$sub_id}", ARRAY_A);

        if (count($rows) > 0) {
            foreach ($rows as $row) {
                $invoice = new Invoice($row);
                array_push($invoices, $invoice);
            }
        }

        return $invoices;
    }


    /**
     * get wppm invoice id
     *
     * @return int|null
     */
    public function getId()
    {
        return intval($this->id);
    }


    /**
     * set wppm invoice id
     *
     * @return self
     */
    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }


    /**
     *
     * @return int
     */
    public function getSubscriptionId()
    {
        return $this->subscription_id;
    }


    /**
     * @param int subscription_id
     *
     * @return self
     */
    public function setSubscriptionId($subscription_id)
    {
        $this->subscription_id = $subscription_id;
        return $this;
    }


    /**
     * Get payment system id: testpay|paypal|stripe
     *
     * @return enum string
     */
    public function getPaymentSystemId()
    {
        return $this->payment_system_id;
    }


    /**
     *
     * @param string payment_system_id
     *
     * @return self
     */
    public function setPaymentSystemId($payment_system_id)
    {
        $this->payment_system_id = $payment_system_id;
        return $this;
    }


    /**
     * Get payment system id: testpay|paypal|stripe
     *
     * @return enum string
     */
    public function getPaymentSystemInvoiceId()
    {
        return $this->payment_system_invoice_id;
    }


    /**
     *
     * @param string payment_system_id
     *
     * @return self
     */
    public function setPaymentSystemInvoiceId($id)
    {
        $this->payment_system_invoice_id = $id;
        return $this;
    }


    /**
     *
     * @return array
     */
    public function getPaymentSystemInvoiceData()
    {
        return maybe_json_decode($this->payment_system_invoice_data, true);
    }


    /**
     * @param any data
     *
     * @return self
     */
    public function setPaymentSystemInvoiceData($data)
    {
        $this->payment_system_invoice_data = maybe_json_encode($data);

        if ($this->getParser()) {
            $this->setAmount($this->getParser()->getAmount());
            $this->setCurrencyCode($this->getParser()->getCurrencyCode());
        }

        return $this;
    }


    /**
     *
     * @return wp_date format date_option
     */
    public function getPaymentDate()
    {
        return $this->getParser()->getPaymentDate();
    }


    /**
     * Get amount
     */
    public function getAmount()
    {
        return number_format($this->amount, 2);
    }

    /**
     * Set amount
     *
     * @return  self
     */
    public function setAmount($amount)
    {
        $this->amount = $amount;

        return $this;
    }

    /**
     * Get currency code
     */
    public function getCurrencyCode()
    {
        return $this->currency_code;
    }

    /**
     * Set currency code
     *
     * @return  self
     */
    public function setCurrencyCode($currency_code)
    {
        $this->currency_code = $currency_code;

        return $this;
    }

    /**
     * Get created timestamp
     */
    public function getCreatedAt()
    {
        return $this->created_at;
    }

    /**
     * Set created timestamp
     *
     * @return  self
     */
    private function setCreatedAt($created)
    {
        $this->created_at = $created;

        return $this;
    }
}
