<?php

namespace WPDM\WPPM\Payment\PayPal;

if (!defined('ABSPATH')) exit;

use DateTime;
use DateTimeInterface;
use Exception;
use Ohanspace\PayPalPHPSdk\Core\PayPalHttpClient;
use Ohanspace\PayPalPHPSdk\Core\SandboxEnvironment;
use Ohanspace\PayPalPHPSdk\Products\ProductsGetRequest;
use Ohanspace\PayPalPHPSdk\Core\ProductionEnvironment;
use Ohanspace\PayPalPHPSdk\Models\BillingCycle;
use Ohanspace\PayPalPHPSdk\Models\Frequency;
use Ohanspace\PayPalPHPSdk\Models\Money;
use Ohanspace\PayPalPHPSdk\Models\Patch;
use Ohanspace\PayPalPHPSdk\Models\PaymentPreferences;
use Ohanspace\PayPalPHPSdk\Models\Plan;
use Ohanspace\PayPalPHPSdk\Models\PricingScheme;
use Ohanspace\PayPalPHPSdk\Models\UpdatePricingSchemeRequest;
use Ohanspace\PayPalPHPSdk\Orders\OrdersCaptureRequest;
use Ohanspace\PayPalPHPSdk\Orders\OrdersGetRequest;
use Ohanspace\PayPalPHPSdk\Plans\PlansCreateRequest;
use Ohanspace\PayPalPHPSdk\Plans\PlansGetRequest;
use Ohanspace\PayPalPHPSdk\Plans\PlansListRequest;
use Ohanspace\PayPalPHPSdk\Plans\PlansPatchRequest;
use Ohanspace\PayPalPHPSdk\Plans\PlansPricingSchemesUpdateRequest;
use Ohanspace\PayPalPHPSdk\Products\ProductsCreateRequest;
use Ohanspace\PayPalPHPSdk\Subscriptions\SubscriptionsCancelRequest;
use Ohanspace\PayPalPHPSdk\Subscriptions\SubscriptionsDetailsGetRequest;
use Ohanspace\PayPalPHPSdk\Subscriptions\SubscriptionsGetTransactionsRequest;
use ParagonIE\Sodium\Core\Curve25519\Fe;
use PayPalHttp\HttpException;
use WPDM\__\Crypt;
use WPDM\WPPM\Invoice\Invoice;
use WPDM\WPPM\Payment\RemotePaymentSystem;
use WPDM\WPPM\Plan\Plan as WPPM_Plan;
use WPDM\WPPM\Plan\PlanComparator;
use WPDM\WPPM\Subscription\Subscription;
use WPDM\WPPM\WPPM;

/**
 * Handles the PayPal Payment System for the application
 *
 *
 *
 */
class PayPalPaymentSystem extends RemotePaymentSystem
{
    /**
     * used to identify the payment system
     */
    protected $payment_system_id = "paypal";

    /**
     * Payment system name
     */
    protected $payment_system_name = "PayPal";

    /**
     * holds singleton instance of the class
     */
    private static $instance;

    /**
     * for paypal js sdk parameter
     */
    private $integration_date = "2020-07-09";

    /**
     * paypal client id
     */
    private $client_id;

    /**
     * paypal client secret
     */
    private $client_secret;

    /**
     * holds the paypal http client
     */
    protected  $client;

    /**
     * holds PayPalWebhookManager
     */
    public $webhook_manager;

    /**
     * PayPalWebhookHandler
     */
    public $webhook_handler;


    /**
     * Get singleton instance of the class
     *
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }


    /**
     * Constructor
     *
     */
    protected function __construct()
    {
        parent::__construct();

        $this->addHooks();
        $this->initClient();
    }


    /**
     * initialize the necessary credentials and paypal http client
     *
     */
    public function initClient()
    {
        $environment = null;

        $this->client_id = $this->getSettingValueFromCurrentMode('client_id');
        $this->client_secret = $this->getSettingValueFromCurrentMode('client_secret');

        if ($this->isTestMode()) {
            $environment = new SandboxEnvironment($this->client_id, $this->client_secret);
        } elseif ($this->isLiveMode()) {
            $environment = new ProductionEnvironment($this->client_id, $this->client_secret);
        }

        if ($environment) {
            $this->client = new PayPalHttpClient($environment);
        }
    }


    /**
     *
     * @return PayPalWebhookManager
     */
    public function getWebhookManager()
    {
        return new PayPalWebhookManager($this);
    }


    /**
     *
     * @return PayPalWebhookHandler
     */
    public function getWebhookHandler()
    {
        return new PayPalWebhookHandler($this);
    }


    /**
     *
     * @return PayPalSubscriptionParser
     */
    public function getSubscriptionParser()
    {
        return new PayPalSubscriptionParser();
    }


    /**
     *
     * @return PayPalInvoiceParser
     */
    public function getInvoiceParser()
    {
        return new PayPalInvoiceParser();
    }


    /**
     * Check if API connection succeed
     *
     * @return boolean
     */
    public function apiConnected()
    {
        if ($this->client) {
            $request = new PlansListRequest();

            try {
                $res = $this->client->execute($request);
                return true;
            } catch (HttpException $e) {
                return false;
            }
        }

        return false;
    }


    /**
     * add hooks
     */
    private function addHooks()
    {
        add_action("wp_enqueue_scripts", array($this, "addStylesAndScripts"));
        add_shortcode("paypal_test", array($this, 'paypal_test'));
    }


    /**
     * add styles and scripts
     *
     */
    public function addStylesAndScripts()
    {
        $script_url = add_query_arg([
            'currency' => wpdmpp_currency_code(),
            'client-id' => $this->client_id,
            'integration-date' => $this->integration_date,
            'vault' => "true"
        ], 'https://www.paypal.com/sdk/js');

        wp_enqueue_script('paypal-js-sdk', esc_url($script_url), [], null, true);
    }


    /**
     * Render Settings
     *
     */
    public function renderSettings()
    {
        $args = [
            'payment_system' => $this
        ];

        $file = __DIR__ . '/settings.php';
        wppm_render_template($file, $args);
    }


    /**
     * Render paypal smaart button interface
     *
     */
    public function renderPaymentInterface(WPPM_Plan $plan)
    {

        try {
            $paypal_plan_id = $this->getOrCreatePlanId($plan);
        } catch (Exception $e) {
            echo $e->getMessage();
            exit;
        }
        // pd($paypal_plan_id);

        $args = [
            'payment_system_id' => $this->getPaymentSystemId(),
            'wppm_plan_id' => $plan->getId(),
            'paypal_plan_id' => $paypal_plan_id
        ];
        $env = $this->isTestMode() ? 'sandbox' : 'live';
        $client_id = $this->client_id;
        $client_secret = $this->client_secret;
        $total = $plan->getInitialAmount();// + $plan->getBillingAmount();
        $plan_name = $plan->getName();
        $order_id = Crypt::encrypt(json_encode(['plan_id' => $plan->getId(), 'user' => get_current_user_id()]));
        $file = __DIR__ . '/payment-interface'.($plan->getBillingPeriodCount() === 0 ? '-single' : '').'.php';
        $args['env'] = $env;
        $args['client_id'] = $client_id;
        $args['client_secret'] = $client_secret;
        $args['total'] = $total;
        $args['plan_name'] = $plan_name;
        $args['order_id'] = $order_id;
        wppm_render_template($file, $args);
    }



    public function processSubscription(Subscription $subscription)
    {
        $data = $subscription->getPaymentSystemSubscriptionData();
        $paypal_subscription_id = wpdm_valueof($data, 'subscriptionID');
        $paypal_order_id = $data['orderID'];
        $paypal_subscription_id = $paypal_subscription_id ?: $paypal_order_id;

        $plan_id = wpdm_query_var('plan_id');
        $plan = WPPM()->plan_manager->getPlan($plan_id);

        $subscription->setPaymentSystemSubscriptionId($paypal_subscription_id);

        if($plan->billing_period_count > 0)
            $subscription->fetchPaymentSystemSubscriptionData();

        $subscription->setSubscriptionStatusActive();
        $subscription->save();
        $invoice = new Invoice();
        $invoice->setSubscriptionId($subscription->getId())
            ->setPaymentSystemId($this->getPaymentSystemId())
            //Paypal subscription do not provide an invoice id, so we are inserting the Order ID instead
            ->setPaymentSystemInvoiceId($paypal_order_id);
        if($plan->billing_period_count > 0)
            $invoice->setPaymentSystemInvoiceData($subscription->getParser()->getLastPaymentData());
        else {

            $invoice->setPaymentSystemInvoiceData(['amount' => ['currency_code' => wpdmpp_currency_code(), 'value' => $plan->initial_amount ]]);
        }
        //{"amount":{"currency_code":"USD","value":"8.0"},"time":"2020-09-07T06:43:16Z"}
        $invoice->save();

        return $subscription->getId();
    }


    /**
     *
     * @return array
     */
    public function getSubscriptionTransactions(string $subscription_id, DateTime $start_time, DateTime $end_time)
    {
        $request = new SubscriptionsGetTransactionsRequest(
            $subscription_id,
            $start_time->format(DateTimeInterface::RFC3339_EXTENDED),
            $end_time->format(DateTimeInterface::RFC3339_EXTENDED)
        );

        $response = $this->getClient()->execute($request);
        return $response->result->transactions ?: [];
    }

    /**
     *
     *
     */
    public function cancelSubscription(Subscription $subscription)
    {
        $paypal_subscription_id = $subscription->getPaymentSystemSubscriptionId();

        $request = new SubscriptionsCancelRequest($paypal_subscription_id);
        $this->client->execute($request);

        $subscription->setPaymentSystemSubscriptionData(null);

        return $subscription;
    }


    /**
     * testing hook. page /paypal-test
     *
     */
    public function paypal_test()
    {
        try {
            $start_time = new DateTime('-2 days');
            $end_time = new DateTime();
            $transactions = $this->getSubscriptionTransactions('I-83UUS82WVBNS', $start_time, $end_time);
            pd($transactions);
        } catch (Exception $e) {
            pd($e->getMessage());
        }
    }


    /**
     * First create product then create plan
     *
     * @return string
     */
    protected function createPlan(WPPM_Plan $wppm_plan)
    {
        $product_id = null;
        //first try create a product
        if ($this->productExists($wppm_plan->getProductId())) {
            $product_id = $wppm_plan->getProductId();
        } else {
            $prod = $this->createProduct($wppm_plan);
            $product_id = $prod->id;
        }

        $paypal_plan = $this->createPayPalPlan($wppm_plan, $product_id);

        return $paypal_plan->id;
    }


    /**
     * Update existing plan
     *
     * @return string
     */
    protected function updatePlan(string $payment_system_plan_id, WPPM_Plan $wppm_plan, PlanComparator $plan_comparator)
    {
        if ($plan_comparator->initialAmountChanged()) {
            $this->updateSetupFee($payment_system_plan_id, $wppm_plan);
        }

        if ($plan_comparator->pricingSchemesChanged()) {
            $this->updatePricingSchemes($payment_system_plan_id, $wppm_plan, $plan_comparator);
        }

        return $payment_system_plan_id;
    }



    /**
     * Create a product in paypal server
     *
     * @return Product
     */
    private function createProduct(WPPM_Plan $wppm_plan)
    {
        $request = new ProductsCreateRequest();
        $request->payPalRequestId($wppm_plan->getProductId());
        $request->body = [
            "id" => $wppm_plan->getProductId(),
            "name" => $wppm_plan->getName(),
            "description" =>  $wppm_plan->getDescription() ?: $wppm_plan->getName(),
            "type" =>  "SERVICE",
            "category" =>  "SOFTWARE",
            "image_url" =>  "https://cdn.wpdownloadmanager.com/wp-content/themes/wpdm5/images/wordpress-download-manager-logo.png", //TODO
            "home_url" => "https://wpdownloadmanager.com/"
        ];

        $res = $this->client->execute($request);
        return $res->result;
    }


    /**
     *
     * @return boolean
     */
    private function productExists($prod_id)
    {
        $request = new ProductsGetRequest($prod_id);
        try {
            $res = $this->client->execute($request);
            return true;
        } catch (Exception $e) {
            return false;
        }
    }


    /**
     * Create a plan
     *
     * @return Plan
     */
    private function createPayPalPlan(WPPM_Plan $wppm_plan, $paypal_product_id)
    {
        //Paypal Plan
        $plan = new Plan();
        $plan->setProduct_id($paypal_product_id);
        $plan->setName($wppm_plan->getName())
            ->setDescription($wppm_plan->getDescription() ?: $wppm_plan->getName())
            ->setStatus("ACTIVE");

        // payment preferences
        $payment_preferences = new PaymentPreferences();
        $payment_preferences->setPayment_failure_threshold(2);

        // Initial setup fee
        if ($wppm_plan->hasInitialFee()) {
            $payment_preferences->setSetup_fee(new Money($wppm_plan->getInitialAmount(), wppm_currency_code()));
        }

        $plan->setPayment_preferences($payment_preferences);


        $sequence = 1;
        //trial fee
        if ($wppm_plan->hasTrial()) {
            $trailCycle = new BillingCycle();
            $trailCycle->setTenure_type("TRIAL")->setSequence($sequence++)->setTotal_cycles(1); // TRIAL cycle is only one time
            $trialPricing =  new PricingScheme(new Money($wppm_plan->getTrialAmount(), wppm_currency_code()));
            $trailCycle->setPricing_scheme($trialPricing);
            $trailCycle->setFrequency(new Frequency($wppm_plan->getTrialPeriodCount(), $wppm_plan->getTrialPeriod()));

            $plan->addBilling_cycle($trailCycle);
        }


        //regular billing
        if ($wppm_plan->hasRecurringBill()) {
            $billingCycle = new BillingCycle();
            $pricingScheme = new PricingScheme(new Money($wppm_plan->getBillingAmount(), wppm_currency_code()));
            $billingCycle->setTenure_type("REGULAR")->setSequence($sequence++);
            $billingCycle->setPricing_scheme($pricingScheme);
            $billingCycle->setFrequency(new Frequency($wppm_plan->getBillingPeriodCount(), $wppm_plan->getBillingPeriod()));
            $billingCycle->setTotal_cycles($wppm_plan->getBillingLimit());

            $plan->addBilling_cycle($billingCycle);
        }


        //Execute HTTP request to the server
        $request = new PlansCreateRequest();
        $request->payPalRequestId(generate_random_string(10));
        $request->body = $plan->toArray();

        //wppm_write_log($request->body);
        $res = $this->client->execute($request);
        //wppm_write_log($res->result);
        return $res->result;
    }


    /**
     * Update the setup fee
     *
     * @return string
     */
    private function updateSetupFee($payment_system_plan_id, WPPM_Plan $wppm_plan)
    {
        $patch = new Patch();
        $patch->setOp("replace");
        $patch->setPath("/payment_preferences/setup_fee");
        $patch->setValue(new Money($wppm_plan->getInitialAmount(), wppm_currency_code()));

        $request = new PlansPatchRequest($payment_system_plan_id);
        $request->body = [$patch->toArray()];

        $this->client->execute($request);
        return $payment_system_plan_id;
    }


    /**
     * Update TRIAL or REGULAR pricing schemes
     * WE CAN ONLY CHANGE A SCHEME IF IT WAS ADDED BEFORE
     * That means yuu cannot set a trial fee if you didnot set it before
     *
     * @return string
     */
    private function updatePricingSchemes($payment_system_plan_id, WPPM_Plan $wppm_plan, PlanComparator $plan_comparator)
    {
        $schemes = [];
        $sequence = 1;

        // TRIAL amount update
        if ($plan_comparator->hasTrialBefore()) {
            $trailSequence = $sequence++;

            if ($plan_comparator->trialAmountChanged()) {
                $trialPricing =  new PricingScheme(new Money($wppm_plan->getTrialAmount(), wppm_currency_code()));

                $trailScheme = new UpdatePricingSchemeRequest();
                $trailScheme->setBilling_cycle_sequence($trailSequence);
                $trailScheme->setPricing_scheme($trialPricing);

                array_push($schemes, $trailScheme->toArray());
            }
        }

        // REGULAR Amount update
        if ($plan_comparator->billingAmountChanged()) {
            $billingPricing = new PricingScheme(new Money($wppm_plan->getBillingAmount(), wppm_currency_code()));

            $billingScheme = new UpdatePricingSchemeRequest();
            $billingScheme->setBilling_cycle_sequence($sequence++);
            $billingScheme->setPricing_scheme($billingPricing);

            array_push($schemes, $billingScheme->toArray());
        }


        $request = new PlansPricingSchemesUpdateRequest($payment_system_plan_id);
        $request->body = ["pricing_schemes" => $schemes];

        // pd($request->body);

        $this->client->execute($request);

        return $payment_system_plan_id;
    }


    /**
     *
     * @return Subscription
     */
    public function getSubscriptionDetails($subscription_id)
    {
        $request = new SubscriptionsDetailsGetRequest($subscription_id);
        $response = $this->getClient()->execute($request);
        return $response->result;
    }
}
