<?php

namespace WPDM\WPPM\Payment;

use Exception;
use WPDM\WPPM\Plan\Plan;
use WPDM\WPPM\Subscription\Subscription;

if (!defined('ABSPATH')) exit;

/**
 * 
 * This is an abstract payment system
 * 
 * All other available payment systems must implement this class
 * 
 * 
 */

abstract class PaymentSystem
{
    /**
     * System id to identify the payment system
     */
    protected $payment_system_id;

    /**
     * Name of the payment system
     */
    protected $payment_system_name;

    /**
     * holds the payment specific settings
     */
    protected $settings;


    /**
     * Render payment interface in checkout
     * 
     * @return void
     */
    public function renderPaymentInterface(Plan $plan)
    {
    }

    /**
     * Cancel a subscription
     * 
     * @return Subscription
     */
    public function cancelSubscription(Subscription $subscription)
    {
        return $subscription;
    }


    /**
     * initialize the settings from wp option
     * 
     */
    protected function __construct()
    {
        $this->initSettings();
    }


    /**
     * Init settings from wp option
     * 
     */
    public function initSettings()
    {
        $settings = get_option($this->getSettingsKey(), []);
        $this->setSettings($settings);
    }


    /**
     * get settings
     * 
     */
    public function getSettings()
    {
        return $this->settings;
    }


    /**
     * Set the payment system specific settings
     * 
     */
    public function setSettings($settings)
    {
        $this->settings = $settings;
        return $this;
    }


    /**
     * 
     * @return boolean
     */
    public function isFree()
    {
        return false;
    }


    /**
     * 
     * @return boolean
     */
    public function isRemotePaymentSystem()
    {
        return $this instanceof RemotePaymentSystem;
    }


    /**
     * Should the payment system show in settings
     * 
     * @return boolean
     */
    public function showInSettings()
    {
        return true;
    }


    /**
     * Save the settings of the payment system
     * 
     * @return boolean
     */
    public function saveSettings($settings)
    {
        update_option($this->getSettingsKey(), $settings);
    }


    /**
     * Get settings key to use in form field name, and to save/get data in wp option
     * 
     * @return string
     */
    public function getSettingsKey()
    {
        return 'wppm_settings_payment_system_' . $this->getPaymentSystemId();
    }


    /**
     * Get a single setting value
     * 
     * @return array|string|null
     */
    public function getSettingValue($key)
    {
        $settings = $this->getSettings();
        return wpdm_valueof($settings, $key);
    }


    /**
     * This is helpful for settings form field name 
     * 
     * @return string
     */
    public function getSettingsFormKey()
    {
        return  WPPM_PAYMENT_SYSTEMS_SETTINGS_DATA_KEY . "[" . $this->getPaymentSystemId() . "]";
    }


    /**
     * Get the payment system id
     * 
     * @return string|Error
     */
    public function getPaymentSystemId()
    {
        if (empty($this->payment_system_id)) {
            throw new Exception("No Payement system id found for " . get_class($this));
        }

        return strtolower($this->payment_system_id);
    }


    /**
     * Get the payment system name
     * 
     * @return string|Error
     */
    public function getPaymentSystemName()
    {
        if (empty($this->payment_system_name)) {
            throw new Exception("No Payement system name found for " . get_class($this));
        }

        return $this->payment_system_name;
    }


    /**
     * Is payment system enabled in global settings
     * 
     * @return boolean
     */
    public function isEnabled()
    {
        $settings = $this->getSettings();

        return isset($settings['enabled']) && $settings['enabled'] == 1 ? true : false;
    }


    /**
     * 
     * @return string
     */
    public function getModeName()
    {
        return strtoupper($this->getSettingValue('mode'));
    }


    /**
     * 
     * @return array|null
     */
    public function getModeSettings()
    {
        $mode = strtolower($this->getModeName()); //test/live
        return $this->getSettingValue($mode);
    }


    /**
     * Get a setting value from current mode test/live
     * 
     * @return string|null
     */
    public function getSettingValueFromCurrentMode($setting_key)
    {
        $mode = strtolower($this->getModeName()); //test/live
        return $this->getSettingValue($mode . '/' . $setting_key);
    }


    /**
     * 
     * @return boolean
     */
    public function isLiveMode()
    {
        $mode = $this->getSettingValue('mode');
        return $mode === 'live';
    }


    /**
     * 
     * @return boolean
     */
    public function isTestMode()
    {
        $mode = $this->getSettingValue('mode');
        return $mode === 'test';
    }


    /**
     * Get the stripe customer id for the current user
     * 
     * @return string|null
     */
    public function getCustomerIdOfCurrentUser()
    {
        $user_id = get_current_user_id();
        $meta_key = $this->getPaymentSystemId() . '_customer_id';
        $customer_id = get_user_meta($user_id, $meta_key, true);
        return $customer_id;
    }


    /**
     * save stripe customer id in user meta
     * 
     * @return boolean|string|Error
     */
    public function saveCustomerIdOfCurrentUser($customer_id)
    {
        $user_id = get_current_user_id();
        $meta_key = $this->getPaymentSystemId() . '_customer_id';
        return update_user_meta($user_id, $meta_key, $customer_id);
    }
}
