<?php

namespace WPDM\WPPM\Payment;

if (!defined('ABSPATH')) exit;

use Exception;
use WPDM\__\__MailUI;
use WPDM\WPPM\Coupon\Coupon;
use WPDM\WPPM\Payment\Manual\ManualPaymentSystem;
use WPDM\WPPM\Payment\Free\FreePaymentSystem;
use WPDM\WPPM\Payment\PayPal\PayPalPaymentSystem;
use WPDM\WPPM\Payment\Stripe\StripePaymentSystem;
use WPDM\WPPM\Payment\TestPay\TestPayPaymentSystem;
use WPDM\WPPM\Plan\Plan;
use WPDM\WPPM\Plan\PlanComparator;
use WPDM\WPPM\Subscription\Subscription;


/**
 * Handles payment related functionality
 *
 *
 */
class PaymentSystemManager
{
    /**
     * holds singleton instance of the class
     */
    private static $instance;

    /**
     * Holds the registered payment systems that added by
     * filter hook WPPM_FILTER_REGISTER_PAYMENT_SYSTEM
     *
     */
    public $registered_payment_systems = [];


    /**
     * Get singleton instance of the class
     *
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }


    /**
     * Constructor
     *
     */
    private function __construct()
    {
        $this->loadDependencies();
        $this->addHooks();
        $this->init();
    }


    /**
     * Load the dependency files
     *
     */
    private function loadDependencies()
    {
    }


    /**
     * add hooks
     *
     */
    private function addHooks()
    {
        add_action('init', array($this, 'actionTask'));

        add_action('wp_ajax_' . WPPM_AJAX_ACTION_SAVE_PAYMENT_SYSTEMS_SETTINGS, array($this, 'savePaymentSystemsSettings'));
        add_action('wp_ajax_' . WPPM_AJAX_ACTION_RENDER_PAYMENT_SYSTEM_INTERFACE, array($this, 'renderPaymentSystemInterface'));
        add_action('wp_ajax_nopriv_' . WPPM_AJAX_ACTION_RENDER_PAYMENT_SYSTEM_INTERFACE, array($this, 'renderPaymentSystemInterface'));
        add_action('wp_ajax_' . WPPM_AJAX_ACTION_PROCESS_SUBSCRIPTION, array($this, 'processSubscription'));
        add_action('wp_ajax_nopriv_' . WPPM_AJAX_ACTION_PROCESS_SUBSCRIPTION, array($this, 'processSubscription'));

        add_action(WPPM_ACTION_AFTER_PLAN_CPT_SAVE, array($this, 'actionAfterPlanCPTSaved'), 10, 2);
        add_action(WPPP_ACTION_PAYMENT_SYSTEM_ERROR, array($this, 'actionHandlePaymentSystemError'), 10, 2);

        add_filter(WPPM_FILTER_REGISTER_GLOBAL_SETTINGS_TAB, array($this, 'registerPaymentSystemSettingsTab'));
    }


    /**
     * registers payemnt systems
     *
     */
    public function init()
    {
        $this->registered_payment_systems = [
            TestPayPaymentSystem::getInstance(),
            FreePaymentSystem::getInstance(),
            ManualPaymentSystem::getInstance(),
            PayPalPaymentSystem::getInstance()
            //StripePaymentSystem::getInstance()
        ];
        if (class_exists('\WPDMPP\Libs\PaymentMethods\WPDM_Stripe')) {
            $this->registered_payment_systems[] = StripePaymentSystem::getInstance();
        }
        $this->registered_payment_systems = apply_filters(WPPM_FILTER_REGISTER_PAYMENT_SYSTEM, $this->registered_payment_systems);
    }


    /**
     * Calback function called after plan cpt saving
     *
     * Pass plan saving responsibility to the ENABLED payment systems
     *
     */
    public function actionAfterPlanCPTSaved(Plan $plan, PlanComparator $plan_comparator)
    {

        $systems = $this->getEnabledPaymentSystems();

        foreach ($systems as $system) {
            if (method_exists($system, 'savePlan')) {
                try {
                    $system->savePlan($plan, $plan_comparator);
                } catch (Exception $e) {
                    do_action(WPPP_ACTION_PAYMENT_SYSTEM_ERROR, $system->getPaymentSystemId(), $e->getMessage());
                }
                //p($saved);
            }
        }
    }


    /**
     * handles payment system errors log
     *
     */
    public function actionHandlePaymentSystemError($payment_system_id, $error_message)
    {
        global $wppm;
        $wppm->alert_manager->addErrorAlert("{$payment_system_id} - " . $error_message);
    }


    /**
     * Process the subscription after payment
     *
     * FROM AJAX CALLBACK
     *
     */
    public function processSubscription()
    {
        $payment_system_id = \wpdm_query_var('payment_system_id');
        $wppm_plan_id = \wpdm_query_var('plan_id');
        $subscription_data =  \wpdm_query_var('payment_system_subscription_data');

        $payment_system = $this->getPaymentSystem($payment_system_id);

        $user_id = is_user_logged_in() ? get_current_user_id() : wpdm_query_var('uid');

        if ($payment_system) {
            try {
                $subscription = new Subscription();

                $subscription->setUserId($user_id)
                    ->setPlanId($wppm_plan_id)
                    ->setPaymentSystemId($payment_system_id)
                    ->setPaymentSystemSubscriptionData($subscription_data);

                $payment_system->processSubscription($subscription);
                $this->notifySubscription($wppm_plan_id);
                wp_send_json(['success']);
                exit();
            } catch (Exception $e) {
                wp_send_json(['errors' => $e->getMessage()]);
                exit();
            }
        }
    }

    public function notifySubscription($wppm_plan_id)
    {

        $ip = wpdm_get_client_ip();
        $user = wp_get_current_user();

        $params['name'] = $user->display_name;
        $params['email'] = $user->user_email;
        $params['user_ip'] = $ip;
        $params['edit_user_btn'] = "<a class='button' style='display:block;margin:10px 0 0;text-decoration: none;text-align:center;' href='" . admin_url('user-edit.php?user_id=' . $user_id) . "'> " . __("Edit User", "download-manager") . " </a>";
        //Include all data from signup form
        foreach ($_POST['wppm_signup_data'] as $key => $value) {
            $params["user_" . $key] = $value;
        }
        $params['plan'] = get_the_title($wppm_plan_id);
        \WPDM\__\Email::send("new-subscription", $params);
    }


    /**
     * Callback wppp task
     *
     * FROM URL CALLBACK
     */
    public function actionTask()
    {
        $wppm_task = wpdm_query_var('wppm_task');
        if ($wppm_task && $wppm_task == 'PAYMENT_SUCCESS') {

            $payment_system_id = wpdm_query_var('payment_system_id');
            $session_id = wpdm_query_var('session_id');

            try {
                $payment_system = $this->getPaymentSystem($payment_system_id);
                $payment_system->processCheckoutSession($session_id);

                return WPPM()->account_manager->redirectToAccountPage();
            } catch (Exception $e) {
                p($e->getMessage());
            }
        }
    }


    /**
     * Get registered payment systems
     *
     */
    public function getRegisteredPaymentSystems()
    {
        return $this->registered_payment_systems;
    }


    /**
     * Return a registered payment system object based on payment_system_id
     *
     * @return object Specific Payment_System
     */
    public function getRegisteredPaymentSystem($id)
    {
        $systems = $this->getRegisteredPaymentSystems();

        return current(array_filter($systems, function ($ps) use ($id) {
            return $ps->getPaymentSystemId() == $id;
        }));
    }


    /**
     * get the Registered Payment System object based on id
     *
     * @return object Payment_System
     */
    public function getPaymentSystem($system_id)
    {
        return $this->getRegisteredPaymentSystem($system_id);
    }


    /**
     * get only the enabled payment methods
     *
     * @return array
     *
     */
    public function getEnabledPaymentSystems()
    {
        $all_systems = $this->getRegisteredPaymentSystems();

        return array_filter($all_systems, function ($system) {
            return $system->isEnabled();
        });
    }


    /**
     *
     * @return [RemotePaymentSystem]
     */
    public function getRemotePaymentSystems()
    {
        return array_filter($this->getRegisteredPaymentSystems(), function ($system) {
            return $system instanceof RemotePaymentSystem;
        });
    }


    /**
     * Get the default payment system id
     *
     */
    public function getDefaultPaymentSystemId()
    {
        //TODO from settings
        return 'testpay';
    }



    /**
     * displays the payment system selection template
     *
     *
     */
    public function renderPaymentSystemSelection()
    {

        $args = [
            'enabled_payment_systems' =>  $this->getEnabledPaymentSystems()
        ];

        $file = wpdm_tpl_path('payment-system-selection.php', get_stylesheet_directory() . '/wppromembership', __DIR__.'/');
        \wppm_render_template($file, $args);
    }


    /**
     * this is called by payment system selection page
     *
     * @return void
     */
    public function renderPaymentSystemInterface()
    {
        $plan_id =  wpdm_query_var('plan_id');
        $payment_system_id = wpdm_query_var('payment_system_id');
        $coupon_code = wpdm_query_var('coupon_code');
        if (empty($plan_id) || empty($payment_system_id)) {
            die('No plan id OR payment system id found.');
        }
        $payment_system = $this->getPaymentSystem($payment_system_id);
        $plan = WPPM()->plan_manager->getPlan($plan_id);

        $coupon = $coupon = Coupon::getByCode($coupon_code);
        if ($plan && $coupon && $coupon->isApplicableInPlan($plan_id)) {
            $plan->applyCoupon($coupon);
        }
        $payment_system->renderPaymentInterface($plan);
        die();
    }


    /**
     * Callback Register payment systems settings tab in global settings page
     *
     */
    public function registerPaymentSystemSettingsTab($tabs)
    {
        $tab = [
            'id' => 'payment-systems',
            'title' => __('Payment Systems', 'wppromembership'),
            'callback' => array($this, 'renderPaymentSystemsSettings'),
            'order' => 2
        ];

        array_push($tabs, $tab);

        return $tabs;
    }


    /**
     * Render Payment systems settings in global settings tab
     *
     */
    public function renderPaymentSystemsSettings()
    {
        $selected_payment_system_id = \wpdm_query_var('payment_system_id');
        $selected_payment_system = $this->getRegisteredPaymentSystem($selected_payment_system_id);


        $payment_systems = array_filter($this->getRegisteredPaymentSystems(), function ($ps) {
            return $ps->showInSettings();
        });

        if (!$selected_payment_system && count($payment_systems)) {
            $selected_payment_system = $payment_systems[0];
        }

        $args = [
            'payment_systems' => $payment_systems,
            'selected_payment_system' => $selected_payment_system
        ];


        //pd($args);

        $file = __DIR__ . '/payment-systems-settings.php';
        wppm_render_template($file, $args);
    }


    /**
     * Save payment system settings
     *
     */
    public function savePaymentSystemsSettings()
    {
        if (isset($_REQUEST[WPPM_PAYMENT_SYSTEMS_SETTINGS_NONCE])) {

            $payment_systems_settings = wpdm_query_var(WPPM_PAYMENT_SYSTEMS_SETTINGS_DATA_KEY) ?: [];

            // pd($payment_systems_settings);

            foreach ($payment_systems_settings as $payment_system_id => $settings) {
                $payment_system = $this->getPaymentSystem($payment_system_id);
                $payment_system->saveSettings($settings);
            }
            wp_send_json(['status' => 'success', 'data' => $settings]);
        }
    }
}
