<?php

namespace WPDM\WPPM\Payment\Stripe;

if (!defined('ABSPATH')) exit;

use Exception;
use Stripe\StripeClient;
use WPDM\WPPM\Invoice\Invoice;
use WPDM\WPPM\Payment\PaymentSystem;
use WPDM\WPPM\Payment\RemotePaymentSystem;
use WPDM\WPPM\Plan\Plan as WPPM_Plan;
use WPDM\WPPM\Plan\PlanComparator;
use WPDM\WPPM\Subscription\Subscription;

/**
 *
 *
 */
class StripePaymentSystem extends RemotePaymentSystem
{
    /**
     * holds singleton instance of the class
     */
    private static $instance;

    /**
     * payment system id
     */
    protected $payment_system_id = "stripe";

    /**
     * payment system name
     */
    protected $payment_system_name = "Stripe";

    /**
     * stripe http client to api call
     */
    protected $client;

    /**
     * Webhook Manager
     */
    protected $webhook_manager;

    /**
     * webhook handler
     */
    protected $webhook_handler;


    /**
     * Get singleton instance of the class
     *
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }


    /**
     * Constructor
     *
     */
    protected function __construct()
    {
        parent::__construct();
        $this->addHooks();
        $this->initClient();
    }


    /**
     * Add hooks
     *
     */
    private function addHooks()
    {
        add_shortcode("stripe_test", array($this, 'stripe_test'));
        add_action("wp_enqueue_scripts", array($this, "addStylesAndScripts"));
    }


    /**
     * Add styles and scripts
     *
     */
    public function addStylesAndScripts()
    {
        $script_url = "https://js.stripe.com/v3/";
        wp_enqueue_script('stripe-js-sdk', esc_url($script_url), [], null, true);
    }


    /**
     * TODO for testing purpose only in shortcode page stripe_test
     *
     */
    public function stripe_test()
    {
        //$this->createProduct();
        // $this->createPrice();
        //$this->createCustomer();
    }


    /**
     * Initialize the stripe http client
     *
     */
    public function initClient()
    {
        $this->secret_key = $this->getSettingValueFromCurrentMode('secret_key');

        if ($this->secret_key) {
            $this->client = new StripeClient($this->secret_key);
        }
    }


    /**
     *
     * @return StripeWebhookManager
     */
    public function getWebhookManager()
    {
        return new StripeWebhookManager($this);
    }


    /**
     *
     * @return StripeWebhookHandler
     */
    public function getWebhookHandler()
    {
        return new StripeWebhookHandler($this);
    }


    /**
     *
     * @return StripeSubscriptionParser
     */
    public function getSubscriptionParser()
    {
        return new StripeSubscriptionParser();
    }


    /**
     *
     * @return StripeInvoiceParser
     */
    public function getInvoiceParser()
    {
        return new StripeInvoiceParser();
    }


    /**
     * Process the checkout session after stripe subscription
     *
     */
    public function processCheckoutSession($session_id)
    {
        $session = $this->client->checkout->sessions->retrieve($session_id, ['expand' => ['subscription', 'subscription.latest_invoice']]);
        $wppm_plan_id = $session->client_reference_id;

        $stripe_subscription = $session->subscription;
        $stripe_latest_invoice = $stripe_subscription->latest_invoice;

        $subscription = new Subscription();
        $subscription->setUserId(get_current_user_id())
            ->setPlanId($wppm_plan_id)
            ->setPaymentSystemId($this->getPaymentSystemId())
            ->setPaymentSystemSubscriptionId($stripe_subscription->id)
            ->setPaymentSystemSubscriptionData($stripe_subscription)
            ->setSubscriptionStatusActive()
            ->save();


        $invoice = new Invoice();
        $invoice->setSubscriptionId($subscription->getId())
            ->setPaymentSystemId($this->getPaymentSystemId())
            ->setPaymentSystemInvoiceId($stripe_latest_invoice->id)
            ->setPaymentSystemInvoiceData($stripe_latest_invoice)
            ->setAmount($invoice->getParser()->getAmount())
            ->setCurrencyCode($invoice->getParser()->getCurrencyCode())
            ->save();

        return $subscription->getId();
    }


    /**
     * Cancel stripe subscription
     *
     * @return Subscription
     */
    public function cancelSubscription(Subscription $subscription)
    {
        $stripeSub = $this->client->subscriptions->cancel(
            $subscription->getPaymentSystemSubscriptionId(),
            []
        );
        $subscription->setPaymentSystemSubscriptionData($stripeSub);
        $subscription->save();
        return $subscription;
    }


    /**
     * Check if API connection succeed
     *
     * @return boolean
     */
    public function apiConnected()
    {
        if ($this->client) {
            try {
                // TODO we are just trying to fetch products to see if the API working
                $products = $this->client->products->all(['limit' => 1]);
                return true;
            } catch (\Exception $e) {
                return false;
            }
        }
        return false;
    }


    /**
     * Render Stripe Settings in global payment settings
     *
     */
    public function renderSettings()
    {
        $args = [
            'payment_system' => $this
        ];

        $file = __DIR__ . '/settings.php';
        wppm_render_template($file, $args);
    }


    /**
     * Render payment interface during checkout
     *
     */
    public function renderPaymentInterface(WPPM_Plan $plan)
    {
        // In stripe price is similar to plan

        try {
            $stripe_price_id = $this->getOrCreatePlanId($plan);

            $stripe_customer_id = $this->getOrCreateCustomerId();

            $session = $this->createCheckoutSession($plan->getId(), $stripe_price_id, $stripe_customer_id);
        } catch (Exception $e) {
            echo $e->getMessage();
            exit;
        }

        $args = [
            'payment_system_id' => $this->getPaymentSystemId(),
            'wppm_plan_id' => $plan->getId(),
            'publishable_key' => $this->getSettingValueFromCurrentMode('publishable_key'),
            'checkout_session_id' => $session->id
        ];


        $file = __DIR__ . '/payment-interface.php';
        wppm_render_template($file, $args);
    }


    /**
     * Create checckout session
     *
     * @return Stripe\Checkout\Session
     */
    private function createCheckoutSession($wppm_plan_id, $stripe_price_id, $stripe_customer_id)
    {
        $success_url = add_query_arg([
            "wppm_task" => "PAYMENT_SUCCESS",
            "payment_system_id" => $this->getPaymentSystemId(),
            "session_id" => "{CHECKOUT_SESSION_ID}"
        ], get_home_url());

        $cancel_url = add_query_arg([
            "wppm_task" => "checkout"
        ], get_home_url());

        $session = $this->client->checkout->sessions->create([
            'customer' => $stripe_customer_id,
            'payment_method_types' => ['card'],
            'client_reference_id' => $wppm_plan_id,
            'line_items' => [[
                'price' => $stripe_price_id,
                'quantity' => 1,
            ]],
            'mode' => 'subscription',
            'success_url' => $success_url,
            'cancel_url' => $cancel_url,
        ]);

        return $session;
    }


    /**
     * Get or create customer id for the current user
     *
     * @return string
     */
    public function getOrCreateCustomerId()
    {
        //first try to get existing customer id from user meta
        $customer_id = $this->getCustomerIdOfCurrentUser();

        if (empty($customer_id)) {
            //Create new customer
            $customer = $this->createCustomer();
            $customer_id = $customer->id;
        }

        return $customer_id;
    }


    /**
     * Create stripe customer
     *
     * @return Customer
     */
    public function createCustomer()
    {
        $email = wp_get_current_user()->user_email;

        $customer = $this->client->customers->create([
            'email' => $email
        ]);

        return $customer;
    }


    /**
     * First create product then create plan
     * stripe price is similar to plan concept
     *
     * @return string
     */
    protected function createPlan(WPPM_Plan $wppm_plan)
    {
        if ($this->productExists($wppm_plan->getProductId())) {
            $product_id = $wppm_plan->getProductId();
        } else {
            $prod = $this->createProduct($wppm_plan);
            $product_id = $prod->id;
        }

        $price = $this->createPrice($wppm_plan, $product_id);

        return $price->id;
    }


    /**
     * Make the current pricing inactive and create new Pricing
     *
     * @return string
     */
    protected function updatePlan($price_id, WPPM_Plan $wppm_plan, PlanComparator $plan_comparator)
    {
        //inactive the previous plan/price
        $old_price = $this->inactivePrice($price_id);
        $new_price = $this->createPrice($wppm_plan, $old_price->product);

        $wppm_plan->savePlanIdOfPaymentSystem($this->getPaymentSystemId(), $new_price->id);
        return $new_price->id;
    }


    /**
     * Create Stripe Product
     *
     * @return Product
     */
    private function createProduct(WPPM_Plan $wppm_plan)
    {

        $prod = $this->client->products->create([
            'id' => $wppm_plan->getProductId(),
            'name' => $wppm_plan->getProductName()
        ]);

        return $prod;
    }


    /**
     * 
     * @return boolean
     */
    private function productExists($product_id)
    {
        try {
            $prod = $this->client->products->retrieve($product_id, []);
            if ($prod && $prod->id) {
                return true;
            }
        } catch (Exception $e) {
        }
        return false;
    }


    /**
     * Create Plan/Pricing
     *
     * @return Price
     */
    private function createPrice(WPPM_Plan $wppm_plan, $stripe_product_id)
    {
        $recurring = [
            'interval' => strtolower($wppm_plan->getBillingPeriod()),
            'interval_count' => $wppm_plan->getBillingPeriodCount()
        ];

        $price = $this->client->prices->create([
            'product' => $stripe_product_id,
            'currency' => WPPM()->currency_code,
            'unit_amount' => $wppm_plan->getBillingAmount() * 100,
            'active' => true,
            'recurring' => $recurring
        ]);

        return $price;
    }


    /**
     * Make a pricing inactiove
     *
     * @return Price
     */
    private function inactivePrice($price_id)
    {
        $price = $this->client->prices->update($price_id, ['active' => false]);
        return $price;
    }


    /**
     *
     * @return subscription
     */
    public function getSubscriptionDetails($sub_id)
    {
        $subscription = $this->client->subscriptions->retrieve($sub_id, ['latest_invoice']);
        return $subscription;
    }
}
