<?php

namespace WPDM\WPPM\Plan;

use CAMPAIGN\INIT\Taxonomy;
use WPDM\libs\CategoryHandler;
use WPDM\WPPM\Coupon\Coupon;

/**
 * Client Consumable Plan class
 *
 * this class handles necessary data transformation from plan cpt, retrieval of a Plan CPT
 *
 *
 */
class Plan
{
    /**
     * Holds the plan meta data
     */
    private $_plan_meta_data;

    /**
     * ID of the plan cpt
     */
    public $id;

    /**
     *
     *
     */
    public $role_id;

    /**
     * Plan name
     */
    public $name = '';

    /**
     * Plan description
     */
    public $description = '';

    /**
     * Confirmation Text
     */
    public $confirmation = '';

    /**
     * Features text
     */
    public $features = '';

    /**
     * Initial amount
     */
    public $initial_amount = 0;

    /**
     * billing amount
     */
    public $billing_amount = 0;

    /**
     * billing period count
     */
    public $billing_period_count = 0;

    /**
     * billing period
     */
    public $billing_period = 'MONTH';

    /**
     * Billing cycle limit, 0 for indefinite
     */
    public $billing_limit = 0;

    /**
     * Trail amount
     */
    public $trial_amount = 0;

    /**
     * trail period count  Days
     */
    public $trial_period_count = 0;

    /**
     * Trail period
     */
    public $trial_period = "DAY";

    /**
     * Is subscription allowed for this plan
     */
    public $allow_subscription = 'YES';

    /**
     *
     * Coupon
     */
    private $coupon;

    /**
     * @var \WP_Term[]
     */
    private $tax_terms;




    /**
     * create the PLAN obj by passing PLAN_CPT in the constructor
     *
     */
    public function __construct($data = null)
    {
        switch (gettype($data)) {
            case 'NULL':
                break;
            case 'object':
                $this->fromPlanCPT($data);
                break;
            default:
        }
    }


    /**
     * transform plan post object to plan object
     *
     */
    private function fromPlanCPT($post)
    {
        $this->id = $post->ID;
        $this->name = $post->post_title;
        $this->description = $post->post_content;

        $meta =  $this->initPlanMetaData();

        if (count($meta) > 0) {

            $meta_properties = [
                'role_id',
                'confirmation',
                'features',
                'initial_amount',
                'billing_amount',
                'billing_limit',
                'billing_period_count',
                'billing_period',
                'trial_amount',
                'trial_period_count',
                'allow_subscription',
                'tax_terms'
            ];

            foreach ($meta_properties as $prop) {
                $this->$prop = $this->getMetaValue($prop) ?: $this->$prop;
            }
        }
    }


    /**
     *
     * @return Plan|null
     */
    public static function get($plan_id)
    {
        return PlanManager::getInstance()->getPlan($plan_id);
    }


    /**
     * fetch meta data and save it locally
     *
     */
    private function initPlanMetaData()
    {
        return $this->_plan_meta_data = get_post_meta($this->getId()) ?: [];
    }


    /**
     * get the plan meta data with defaults merged
     *
     */
    public function getMetaData()
    {
        return $this->_plan_meta_data;
    }


    /**
     * Get the a plan meta value
     *
     * @param key without prefixing
     *
     * @return mix|null
     */
    public function getMetaValue($key)
    {
        $meta_data = $this->getMetaData();
        $meta_key = WPPM_PLAN_META_PREFIX . $key;

        if (isset($meta_data[$meta_key])) {
            return maybe_unserialize($meta_data[$meta_key][0]);
        }

        return null;
    }


    /**
     *
     * @return string|null
     */
    public function getPlanIdOfPaymentSystem($system_id)
    {
        $key = $system_id . '_plan_id';
        return $this->getMetaValue($key);
    }


    /**
     * Saves the plan id of a payment system after created
     *
     * @return boolean
     */
    public function savePlanIdOfPaymentSystem($payment_system_id, $payment_system_plan_id)
    {
        $key = WPPM_PLAN_META_PREFIX . $payment_system_id . '_plan_id'; //__wppm_plan_paypal_plan_id
        $meta_id = update_post_meta(
            $this->getId(),
            $key,
            $payment_system_plan_id
        );

        return is_int($meta_id) ? true : false;
    }


    //=============================== Getters =============================================================

    /**
     * Get the plan id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }


    /**
     * Get the slug
     *
     * @return string
     */
    public function getSlug()
    {
        return 'wppm_' . $this->getId();
    }


    /**
     * Get the product id
     * Product id is useful in payment systems
     * We are using the pattern wppm_plan_product_{plan_id}
     *
     * @return string
     */
    public function getProductId()
    {
        return 'wppm_plan_product_' . $this->getId();
    }


    /**
     * Get product name
     *
     * @return string
     */
    public function getProductName()
    {
        return $this->getName();
    }


    /**
     *
     * @return string
     */
    public function getRoleId()
    {
        return $this->role_id ?: WPPM_ROLE_PREFIX . $this->getId(); //wppm_123
    }


    /**
     *
     * @return string
     */
    public function getRoleName()
    {
        return $this->getName();
    }


    /**
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }


    /**
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }


    /**
     *
     * @return string
     */
    public function getConfirmation()
    {
        return $this->confirmation;
    }


    /**
     * get features
     *
     * @return string
     */
    public function getFeatures()
    {
        return $this->features;
    }


    /**
     * Get initial amount (setup fee)
     *
     * @return float
     */
    public function getInitialAmount()
    {
        return number_format((float)$this->initial_amount, 2);
    }


    /**
     *
     * @return boolean
     */
    public function hasInitialFee()
    {
        return $this->getInitialAmount() > 0;
    }


    /**
     *
     * @return float
     */
    public function getBillingAmount()
    {
        return number_format((float)$this->billing_amount, 2);
    }


    /**
     *
     * @return int
     */
    public function getBillingLimit()
    {
        return intval($this->billing_limit);
    }


    /**
     *
     * @return int
     */
    public function getBillingPeriodCount()
    {
        return intval($this->billing_period_count);
    }


    /**
     *
     * @return enum string DAY WEEK MONTH YEAR
     */
    public function getBillingPeriod()
    {
        return $this->billing_period;
    }


    /**
     *
     * @return float
     */
    public function getTrialAmount()
    {
        return number_format($this->trial_amount, 2);
    }


    /**
     *
     * @return int
     */
    public function getTrialPeriodCount()
    {
        return intval($this->trial_period_count);
    }


    /**
     *
     * @return string
     */
    public function getTrialPeriod()
    {
        return $this->trial_period;
    }


    /**
     *
     * @return boolean
     */
    public function hasTrial()
    {
        return $this->getTrialPeriodCount() > 0;
    }


    /**
     *
     * @return string YES|NO
     */
    public function getAllowSubscription()
    {
        return $this->allow_subscription;
    }

    // ========================================End Getters ==========================================================


    /**
     * Return true if initial, billing, and trail amount are all zero
     *
     * @return boolean
     */
    public function isFree()
    {
        if (
            $this->getInitialAmount() <= 0 &&
            $this->getBillingAmount() <= 0 &&
            $this->getTrialAmount() <= 0
        )
            return true;
        else
            return false;
    }


    /**
     * return true if no recurring charge
     *
     * @return boolean
     */
    public function isLifeTime()
    {
        return $this->getBillingPeriodCount() === 0;
    }


    /**
     * return true if trial limit is set to more than zero
     *
     * @return boolean
     */
    public function isTrial()
    {
        return $this->getTrialPeriodCount() > 0;
    }


    /**
     *
     * @return boolean
     */
    public function hasRecurringBill()
    {
        return $this->getBillingPeriodCount() > 0 &&
            !empty($this->getBillingPeriod()) &&
            ($this->getBillingAmount() > 0);
    }


    /**
     *
     * @return boolean
     */
    public function isSubscriptionAllowed()
    {
        return $this->allow_subscription === 'YES';
    }


    /**
     * Get features parsed as list item to display in catalogue
     *
     * @return htmlstring
     */
    public function getFeaturesAsLi()
    {
        $html = '';
        $features = preg_split("/\\r\\n|\\r|\\n/", $this->features);
        foreach ($features as $feature) {
            $feature = preg_replace("/\[\[([^\]]+)\]\]/", "<i class='fa \$1'></i>", $feature);
            if ($feature != '')
                $html .= '<li class="list-group-item d-flex justify-content-between align-items-center">' . stripslashes($feature) . '</li>';
        }

        return $html;
    }


    /**
     *
     * @return htmlstring
     */
    public function getBillingText()
    {

        global  $wpmp_currency_symbol;

        $text = "";
        if ($this->isFree()) {
            $text = __('Free', 'wppromembership');
        } elseif ($this->isLifeTime()) {

            $text =  "<span class='wppm-lifetime-plan'>" .
                sprintf(
                    __('<span><span class="cursym">%s</span>%s</span>', 'wppromembership'),
                    $wpmp_currency_symbol,
                    $this->getInitialAmount()
                ) .
                "</span>  for Life Time";
        } elseif ($this->getBillingAmount() != 0.00) {


            if ($this->getBillingLimit() > 1) {

                if ($this->getBillingPeriodCount() == 1) {

                    $text = sprintf(
                        __('<span class="cursym">%s</span>%s / %s for %d more %s', 'Recurring payment in cost text generation. E.g. $5 / month for 2 more payments.', 'wppromembership'),
                        $wpmp_currency_symbol,
                        $this->getBillingAmount(),
                        $this->getBillingPeriodText(1),
                        $this->getBillingLimit(),
                        $this->getBillingPeriodText($this->getBillingLimit())
                    );
                } else {

                    $text = sprintf(
                        __('<span class="cursym">%s</span>%s / %d %s for %d more %s', 'Recurring payment in cost text generation. E.g., $5 / 2 months for 2 more payments.', 'wppromembership'),
                        $wpmp_currency_symbol,
                        $this->getBillingAmount(),
                        $this->getBillingPeriodCount(),
                        $this->getBillingPeriodText(),
                        $this->getBillingLimit(),
                        $this->getBillingPeriodText($this->getBillingLimit())
                    );
                }
            } elseif ($this->getBillingLimit() == 1) {
                $text =  sprintf(
                    __('<span class="cursym">%s</span>%s after %d %s', 'Recurring payment in cost text generation. E.g. $5 after 2 months.', 'wppromembership'),
                    $wpmp_currency_symbol,
                    $this->getBillingAmount(),
                    $this->getBillingPeriodCount(),
                    $this->getBillingPeriodText()
                );
            } else {
                if ($this->getBillingPeriodCount() == 1) {
                    $text =  sprintf(
                        __('<span class="cursym">%s</span>%s / %s', 'Recurring payment in cost text generation. E.g. $5 / month.', 'wppromembership'),
                        $wpmp_currency_symbol,
                        $this->getBillingAmount(),
                        $this->getBillingPeriodText()
                    );
                } else {
                    $text = sprintf(
                        __('<span class="cursym">%s</span>%s / %d %s', 'Recurring payment in cost text generation. E.g., $5 / 2 months.', 'wppromembership'),
                        $wpmp_currency_symbol,
                        $this->getBillingAmount(),
                        $this->getBillingPeriodCount(),
                        $this->getBillingPeriodText()
                    );
                }
            }
        }

        return $text;
    }


    /**
     *
     * @return string
     */
    public function getCostText()
    {
        global $wpmp_currency_symbol;

        $text = "";

        if ($this->isFree()) $text = "Free membership plan";
        else if ($this->isLifeTime()) $text = "Subscribe for life time";
        else {
            //trial part
            if ($this->getTrialPeriodCount() > 0) {
                if ($this->getTrialAmount() == 0.00) {
                    if ($this->getTrialPeriodCount() == 1) {
                        $text .= ' ' . _x('Your first day is Free.', 'Trial payment in cost text generation.', 'wppromembership');
                    } else {
                        $text .= ' ' . sprintf(
                            _x('Your first %d days are Free.', 'Trial payment in cost text generation.', 'wppromembership'),
                            $this->getTrialPeriodCount()
                        );
                    }
                } else {
                    if ($this->getTrialPeriodCount() == 1) {
                        $text .= ' ' . sprintf(
                            _x('Your first payment will cost %s.', 'Trial payment in cost text generation.', 'wppromembership'),
                            $wpmp_currency_symbol . $this->getTrialAmount()
                        );
                    } else {
                        $text .= ' ' . sprintf(
                            _x('Your first %d days will cost %s.', 'Trial payment in cost text generation. E.g. ... first 2 payments will cost $5', 'wppromembership'),
                            $this->getTrialPeriodCount(),
                            $wpmp_currency_symbol . $this->getTrialAmount()
                        );
                    }
                }
            }

            //initial payment
            if ($this->getTrialPeriodCount() > 0) {
                $text .= " After trail period, ";
            }
            $text .= sprintf(
                _x('<strong>%s</strong> for first %d %s', 'Initial payment in cost text generation.', 'wppromembership'),
                $wpmp_currency_symbol . ((double)$this->getInitialAmount()+(double)$this->getBillingAmount()),
                $this->getBillingPeriodCount(),
                $this->getBillingPeriodText()
            );

            //recurring part
            if ($this->getBillingAmount() != 0.00) {
                if ($this->getBillingLimit() > 1) {
                    if ($this->getBillingPeriodCount() == 1) {
                        $text .= sprintf(
                            __(' and then <strong>%s per %s for %d more %s</strong>.', 'Recurring payment in cost text generation. E.g. $5 every month for 2 more payments.', 'wppromembership'),
                            $wpmp_currency_symbol . $this->getBillingAmount(),
                            $this->getBillingPeriodText(),
                            $this->getBillingLimit(),
                            $this->getBillingPeriodText($this->getBillingLimit())
                        );
                    } else {
                        $text .= sprintf(
                            __(' and then <strong>%s every %d %s for %d more %s</strong>.', 'Recurring payment in cost text generation. E.g., $5 every 2 months for 2 more payments.', 'wppromembership'),
                            $wpmp_currency_symbol . $this->getBillingAmount(),
                            $this->getBillingPeriodCount(),
                            $this->getBillingPeriodText(),
                            $this->getBillingLimit(),
                            $this->getBillingPeriodText($this->getBillingLimit())
                        );
                    }
                } elseif ($this->getBillingLimit() == 1) {
                    $text .= sprintf(
                        __(' and then <strong>%s after %d %s</strong>.', 'Recurring payment in cost text generation. E.g. $5 after 2 months.', 'wppromembership'),
                        $wpmp_currency_symbol . $this->getBillingAmount(),
                        $this->getBillingPeriodCount(),
                        $this->getBillingPeriodText()
                    );
                } else {
                    if ($this->getBillingPeriodCount() == '1') {
                        $text .= sprintf(
                            __(' and then <strong>%s per %s</strong>.', 'Recurring payment in cost text generation. E.g. $5 every month.', 'wppromembership'),
                            $wpmp_currency_symbol . $this->getBillingAmount(),
                            $this->getBillingPeriodText()
                        );
                    } else {
                        $text .= sprintf(
                            __(' and then <strong>%s every %d %s</strong>.', 'Recurring payment in cost text generation. E.g., $5 every 2 months.', 'wppromembership'),
                            $wpmp_currency_symbol . $this->getBillingAmount(),
                            $this->getBillingPeriodCount(),
                            $this->getBillingPeriodText()
                        );
                    }
                }
            } else
                $text .= '.';
        }

        $text = apply_filters("wppp_plan_cost_text", $text, $this);
        return $text;
    }


    /**
     *
     * @return string
     */
    private function getBillingPeriodText($number = null)
    {
        $number = $number ?: $this->getBillingPeriodCount();

        if ($this->getBillingPeriod() == "DAY")
            return _n("Day", "Days", $number, "wppromembership");
        elseif ($this->getBillingPeriod() == "WEEK")
            return _n("Week", "Weeks", $number, "wppromembership");
        elseif ($this->getBillingPeriod() == "MONTH")
            return _n("Month", "Months", $number, "wppromembership");
        elseif ($this->getBillingPeriod() == "YEAR")
            return _n("Year", "Years", $number, "wppromembership");
    }


    /**
     *
     * @return htmlstring
     */
    public function getTrialText()
    {
        global  $wpmp_currency_symbol;

        $trial_text =  '';
        if ($this->isTrial()) {

            if ($this->getTrialAmount() == 0.00) {
                if ($this->getTrialPeriodCount() == 1) {
                    $trial_text .= _x('Your first day is Free.', 'Trial payment in cost text generation.', 'wppromembership');
                } else {
                    $trial_text .= sprintf(_x('Your first %d days are Free.', 'Trial payment in cost text generation.', 'wppromembership'), $this->getTrialPeriodCount());
                }
            } else {
                if ($this->getTrialPeriodCount() == 1) {
                    $trial_text .= sprintf(
                        ' ' . _x('Your first day will cost <span class="cursym">%s</span>%s.', 'Trial payment in cost text generation.', 'wppromembership'),
                        $wpmp_currency_symbol,
                        $this->getTrialAmount()
                    );
                } else {
                    $trial_text .= sprintf(
                        ' ' . _x('Your first %d days will cost <span class="cursym">%s</span>%s.', 'Trial payment in cost text generation. E.g. ... first 2 payments will cost $5', 'wppromembership'),
                        $this->getTrialPeriodCount(),
                        $wpmp_currency_symbol,
                        $this->getTrialAmount()
                    );
                }
            }

            //echo $trial_text;

        }
        if ($this->getInitialAmount() > 0) {
            if ($trial_text != '' && $this->getInitialAmount() > 0) {
                $trial_text .=  __(" After trail period, ", 'wppromembership');
            }
            $trial_text .= sprintf(
                _x('<strong><span class="cursym">%s</span>%s</strong> for first %d %s', 'Initial payment in cost text generation.', 'wppromembership'),
                $wpmp_currency_symbol,
                $this->getInitialAmount(),
                $this->getBillingPeriodCount(),
                $this->getBillingPeriodText()
            );
            //echo $initialText;
        }

        return $trial_text;
    }

    /**
     * Get the active subsciber count in this plan
     * @return int
     */
    public function getActiveSubscibersCount()
    {
        $users = count_users();
        return (int)wpdm_valueof($users, "avail_roles/" . $this->getRoleId());
    }


    /**
     * Apply coupon to get a discounted pricing of the plan
     * @param Coupon $coupon
     *
     * @return self
     */
    public function applyCoupon(Coupon $coupon)
    {
        $this->coupon = $coupon;

        if ($coupon->isPercentDiscount()) {
            $payable_percent = ((100 - $coupon->getDiscountAmount()) / 100);
            if ($payable_percent < 0) $payable_percent = 0;

            if ($this->getInitialAmount() > 0) {
                $this->initial_amount = $payable_percent * $this->getInitialAmount();
            }

            if ($this->getBillingAmount() > 0) {
                $this->billing_amount = $payable_percent * $this->getBillingAmount();
            }

            if ($this->getTrialAmount() > 0) {
                $this->trial_amount = $payable_percent * $this->getTrialAmount();
            }
        } else if ($coupon->isFixedDiscount()) {
            //
            $d = $coupon->getDiscountAmount();
            $this->initial_amount = $this->getInitialAmount() > $d ? $this->getInitialAmount() - $d : 0;
            $this->billing_amount = $this->getBillingAmount() > $d ? $this->getBillingAmount() - $d : 0;
            $this->trial_amount = $this->getTrialAmount() > $d ? $this->getTrialAmount() - $d : 0;
        }

        return $this;
    }


    /**
     *
     * @return boolean
     */
    public function isCouponApplied()
    {
        return $this->coupon != null ? true : false;
    }


    /**
     *
     * @return string
     */
    public function getCheckoutUrl()
    {
        $general_settings = get_option(WPPM_GENERAL_SETTINGS_KEY);
        $general_settings = maybe_unserialize($general_settings);
        return add_query_arg(['wppm_task' => 'checkout', 'plan_id' => $this->getId()], get_permalink($general_settings['wppm_plans_page']));
    }


    /**
     *
     * @return \WP_Term[]
     */
    public function getTerms($taxonomy = '')
    {
        //$taxonomies = PlanCPT::getInstance()->getAttachableTaxonomyNames();
        //return wp_get_post_terms($this->getId(), $taxonomies);

        // For download manager category ( wpdmcategory ),
        // get selected terms from wpdm access control meta
        if($taxonomy === 'wpdmcategory') {
            $role = $this->getRoleId();
            $selected_terms = [];
            $terms = get_terms(['taxonomy' => $taxonomy, 'hide_empty' => false]);
            foreach ($terms as $term) {
                $roles = WPDM()->categories->getAllowedRoles($term->term_id);

                if(in_array($role, $roles))
                    $selected_terms[] = $term->term_id;
            }
            return $selected_terms;
        }
        // For all other taxonomies
        $terms = $this->tax_terms;
        if($taxonomy !== '')
            $terms = wpdm_valueof($terms, $taxonomy);
        if(!is_array($terms)) $terms = [];
        return $terms;
    }

    function isLifeTimePlan($plan_id = null) {
        $plan_id = $plan_id ?: $this->id;
        return !(int)get_post_meta($plan_id, '__wppm_plan_billing_period_count', true);
    }
}
