<?php

namespace WPDM\WPPM\Plan;

if (!defined('ABSPATH')) exit;



class PlanManager
{
    /**
     * holds singleton instance of the class
     */
    private static $instance;


    public $plan_cpt;



    /**
     * private constructor of the class
     *
     */
    private function __construct()
    {
        $this->loadDependencies();
        $this->addHooks();
    }


    /**
     * get the singleton instance of the class
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }

        return self::$instance;
    }


    /**
     * add hooks
     */
    public function addHooks()
    {
        add_shortcode("wppm_plans", array($this, "renderMembershipPlans"));
        add_shortcode("wppm_membership_plans", array($this, "renderMembershipPlans"));
    }


    /**
     * Loads dependency files
     *
     */
    public function loadDependencies()
    {

        $this->plan_cpt = PlanCPT::getInstance();
    }


    /**
     * call back for shortcade wppm_membership_plans
     *
     * @param array
     * ids: optional, use this parameter when you want to show selective plans only or change order of display, value should be plan ids separated by comma
     * cols: the default value of cols is 4. It indicates the number of plans should be shown per row.
     * desc: the default value of desc is 0. If set to 1 plan description will be shown.
     * featured: to feature a plan set the plan id as featured value.
     * style: change plan table styles, use any of the following values : default, smooth, card
     * featured_color: change plan table styles, use any of the following values: primary, success, info, danger
     * planurl: if you want to use a different page to checkout, create a page with the sample shortcode and use that page url here
     */
    public function renderMembershipPlans($params = [])
    {
        global $wppm;
        $checkout = isset($_REQUEST['wppm_task']) && $_REQUEST['wppm_task'] == 'checkout';
        if ($checkout) {
            return $wppm->checkout_manager->renderCheckout();
        }


        $filters = [
            'ids' => isset($params['ids']) && !empty($params['ids']) ? explode(",", $params['ids']) : []
        ];

        $plans = $this->getPlans($filters);
        //p($plans);

        $args = [
            'params' => $params,
            'plans' => $plans,
            'active_plan_ids' => $wppm->subscription_manager->getActivePlanIdsOfCurrentUser(),
            'cols' => isset($params['cols']) ? 12 / $params['cols'] : 4,
            'show_description' => isset($params['desc']) && $params['desc'] == 1,
            'plan_url' => isset($params['planurl']) ? $params['planurl'] : null,
        ];

        $style = isset($params['style']) ? sanitize_text_field($params['style']) : 'default';
        $style = str_replace(array('.', '/'), '', $style);

        //$pref = \WPDM\__\Crypt::encrypt($params);
        if (count($plans) > 0) {
            $file = wppm_tpl_path("plan-catalogue/{$style}.php", __DIR__ );
            return wppm_render_template($file, $args, false);
        }
    }


    /**
     * renders the plan selection radios
     *
     */
    public function renderPlanSelection()
    {
        global $wppm;


        $args = [
            'selected_plan_id' => \wpdm_query_var('plan_id'),
            'plans' => $this->getSubscribablePlans(),
            'active_plan_ids' => $wppm->subscription_manager->getActivePlanIdsOfCurrentUser()
        ];

        $file = wpdm_tpl_path('plan-selection.php', get_stylesheet_directory() . '/wppromembership', __DIR__.'/');
        \wppm_render_template($file, $args);
    }


    /**
     * @param filters ['ids'/ 'limit' / 'meta']
     *
     * @return array [flat Plan obj]
     *
     */
    public function getPlans($filters = [])
    {
        $plan_posts = $this->plan_cpt->getPlanPosts($filters);

        $plans = [];
        foreach ($plan_posts as $post) {
            $plan = new Plan($post);
            array_push($plans, $plan);
        }

        return $plans;
    }


    /**
     * Get plans by provided plan cpt ids
     *
     * @param ids array [post_id]
     *
     * @return array [Plan]
     */
    public function getPlansByIds($ids = [])
    {
        $filters['ids'] = $ids;
        return $this->getPlans($filters);
    }


    /**
     * Get plan data using wp role ID
     * @param $role_id
     * @return Plan
     */
    public function getPlanByRoleId($role_id)
    {
        $plan_id = str_replace(WPPM_ROLE_PREFIX, "", $role_id);
        return $this->getPlan($plan_id);
    }


    /**
     *
     * @return array [Plan]
     */
    public function getSubscribablePlans()
    {
        return array_filter($this->getPlans(), function ($p) {
            return $p->isSubscriptionAllowed();
        });
    }


    /**
     *
     * @return Plan|null
     */
    public function getPlan($plan_id)
    {
        $post = $this->plan_cpt->getPlanPost($plan_id);
        return $post ? new Plan($post) : null;
    }

    /**
     * @return false|string|\WP_Error
     */
    public function getPlansPage()
    {
        $general_settings = get_option(WPPM_GENERAL_SETTINGS_KEY);
        $general_settings = maybe_unserialize($general_settings);
        return get_permalink($general_settings['wppm_plans_page']);
    }
}
