<?php

namespace WPDM\WPPM\Role;

use WPDM\WPPM\Plan\Plan;
use WPDM\WPPM\Plan\PlanComparator;
use WPDM\WPPM\Subscription\Subscription;

if (!defined('ABSPATH')) exit;


class RoleManager
{
    /**
     * holds singleton instance of the class
     */
    private static $instance;


    /**
     * private constructor of the class
     * so this class cannot be created outside
     *
     */
    private function __construct()
    {
        $this->loadDependencies();
        $this->addHooks();
    }


    private function loadDependencies()
    {
    }


    public function addHooks()
    {
        add_action(WPPM_ACTION_AFTER_PLAN_CPT_SAVE, array($this, 'actionAddUserRole'), 11, 2);
        add_action(WPPM_ACTION_AFTER_SUBSCRIPTION_SAVED, array($this, 'actionAfterSubscriptionSaved'));
        add_action(WPPM_ACTION_AFTER_SUBSCRIPTION_SAVED, array($this, 'actionAfterSubscriptionSaved'));
        add_filter("editable_roles", array($this, 'hideWPPMRoles'));
    }


    /**
     * get the singleton instance of the class
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }

        return self::$instance;
    }


    /**
     * Add Plan role if not exists after saving a plan
     *
     */
    public function actionAddUserRole(Plan $plan, PlanComparator $plan_comparator)
    {
        $role = get_role($plan->getRoleId());

        if (!$role) {
            add_role($plan->getRoleId(), $plan->getRoleName());
        }
    }


    /**
     * Assign user to the subscribed plan role
     *
     */
    public function actionAfterSubscriptionSaved(Subscription $sub)
    {
        $user = get_user_by('id', $sub->getUserId());
        $plan = $sub->getPlan();
        $role_id = $plan->getRoleId();

        if ($sub->isActive()) {

            $user->add_role($role_id);
            do_action('wppm_after_user_role_assigned', $role_id, $user);
        } else {

            $user->remove_role($role_id);
            do_action('wppm_after_user_role_removed', $role_id, $user);
        }
    }


    /**
     * Removes the role after a subscription is canceled
     *
     */
    public function actionAfterSubscriptionCanceled(Subscription $sub)
    {
        $plan = WPPM()->plan_manager->getPlan($sub->getPlanId());
        $role_id = $plan->getRoleId();
        $user = get_user_by('id', $sub->getUserId());
        $user->remove_role($role_id);
    }

    /**
     * Returns role ids for membership plans
     * @return array
     */
    public function getRoleIds()
    {
        $plans = WPPM()->plan_manager->getPlans();
        $roles = [];
        foreach ($plans as $plan){
            $roles[] = $plan->getRoleId();
        }
        return $roles;
    }

    /**
     * Returns roles for membership plans
     * @return array
     */
    public function getRoles()
    {
        $plans = WPPM()->plan_manager->getPlans();
        $roles = [];
        foreach ($plans as $plan){
            $roles[$plan->getRoleId()] = $plan->getName();
        }
        return $roles;
    }

    function hideWPPMRoles($roles)
    {
		global $pagenow;
		if($pagenow === 'options-general.php') return $roles;
        $wppm_roles = $this->getRoleIds();
        foreach ($wppm_roles as $role) {
            if(isset($roles[$role])) {
                unset($roles[$role]);
            }
        }
        return $roles;
    }
}
