<?php

namespace WPDM\WPPM\Settings;

if (!defined('ABSPATH')) exit;



class SettingsManager
{

    /**
     * holds singleton instance of the class
     */
    private static $instance;

    private $settings_tabs = [];

    private $default_values = [
        'nonmembertext' => 'Restricted contents! You must subscribe to !!levels!! plan to get access.',
        'notloggedintext' => 'Restricted contents! You must subscribe to !!levels!! plan to get access.',
    ];


    /**
     * private constructor of the class
     * so this class cannot be created outside
     *
     */
    private function __construct()
    {
        $this->loadDependencies();
        $this->addHooks();
    }


    /**
     * load dependency files
     *
     */
    private function loadDependencies()
    {
    }


    /**
     * Add hooks
     *
     */
    public function addHooks()
    {
        add_action("wp_enqueue_scripts", array($this, "addStylesAndScripts"));
        add_action("wp_ajax_wppm_save_general_settings", array($this, "saveSettings"));
        add_filter(WPPM_FILTER_REGISTER_GLOBAL_SETTINGS_TAB, array($this, 'registerGeneralSettingsTab'));
    }


    /**
     * Add styles and scripts
     *
     */
    public function addStylesAndScripts()
    {
        //wp_enqueue_style("bootstrap-select", "https://cdnjs.cloudflare.com/ajax/libs/bootstrap-select/1.13.1/css/bootstrap-select.css");
        //wp_enqueue_script("bootstrap-select-js", "https://cdnjs.cloudflare.com/ajax/libs/bootstrap-select/1.13.1/js/bootstrap-select.min.js");
    }


    /**
     * get the singleton instance of the class
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }

        return self::$instance;
    }


    /**
     * Render global settings page
     *
     *
     */
    public function renderGlobalSettingsPage()
    {
        $settings_tabs = apply_filters(WPPM_FILTER_REGISTER_GLOBAL_SETTINGS_TAB, $this->settings_tabs);

        // sort the tabs based on order
        usort($settings_tabs, function ($a, $b) {
            return $a['order'] > $b['order'];
        });


        $tab_ids = array_column($settings_tabs, 'id');

        $default_tab_id = $settings_tabs[0]['id'];
        $selected_tab_id = \wpdm_query_var('tab');

        $selected_tab_id = in_array($selected_tab_id, $tab_ids) ? $selected_tab_id : $default_tab_id;

        $selected_tab = current(array_filter($settings_tabs, function ($v) use ($selected_tab_id) {
            return $v['id'] == $selected_tab_id;
        }));



        $args = [
            'settings_tabs' => $settings_tabs,
            'selected_tab' => $selected_tab
        ];

        $file = __DIR__ . '/global-settings-container.php';
        wppm_render_template($file, $args);
    }


    /**
     * Calls global settings tab content
     *
     *
     */
    public function renderSettingsTabContent($tab)
    {
        $callback = $tab['callback'];
        $obj = $callback[0];
        $method = $callback[1];
        return $obj->{$method}();
    }


    /**
     * Callback Register General settings tab in global settings page
     *
     */
    public function registerGeneralSettingsTab($tabs)
    {
        $tab = [
            'id' => 'general-settings',
            'title' => __('General Settings', WPPM_TEXTDOMAIN),
            'callback' => array($this, 'renderGeneralSettings'),
            'order' => 1
        ];

        array_push($tabs, $tab);

        return $tabs;
    }


    /**
     * Render General Settings
     *
     */
    public function renderGeneralSettings()
    {
        $general_settings = get_option(WPPM_GENERAL_SETTINGS_KEY);
        $post_types = get_post_types(['public' => true], 'objects');
        $ignore_post_types = ['wppm_plan', 'wpdmpro', 'wpdm', 'attachment', 'elementor_library'];
        $valid_post_types = [];

        foreach ($post_types as $post_type){
            if(!in_array($post_type->name, $ignore_post_types)){
                $valid_post_types[] = $post_type;
            }
        }

        $taxonomies = get_taxonomies(['public' => true], 'objects');
        $ignore_taxonomies = ['post_tag', 'post_format'];
        $valid_taxonomies = [];
        foreach ($taxonomies as $taxonomy){
            if(!in_array($taxonomy->name, $ignore_taxonomies)){
                $valid_taxonomies[] = $taxonomy;
            }
        }
        $args = [
            'settings' => $general_settings,
            'post_types' => $valid_post_types,
            'guarded_post_types' => $this->getGuardedPostTypes(),
            'taxonomies' => $valid_taxonomies,
            'guarded_taxonomies' => $this->getGuardedTaxonomies()
        ];

        $file = __DIR__ . '/tabs/general-settings.php';
        wppm_render_template($file, $args);
    }


    /**
     *
     * @return string[]
     */
    public function getGuardedPostTypes()
    {
        $settings = get_option(WPPM_GENERAL_SETTINGS_KEY);
        $default = ['post', 'page'];
        $guarded_post_types = isset($settings['is_guarded']) ? wpdm_valueof($settings, 'guarded_post_types', ['default' => []]) : $default;
        return $guarded_post_types;
    }

    /**
     *
     * @return string[]
     */
    public function getGuardedTaxonomies()
    {
        $settings = get_option(WPPM_GENERAL_SETTINGS_KEY);
        $default = ['category', 'post_tag'];
        $guarded_taxonomies = isset($settings['is_guarded']) ? wpdm_valueof($settings, 'guarded_taxonomies', ['default' => []]) : $default;
        return $guarded_taxonomies;
    }



    /**
     * Save General Settings
     *
     */
    private function saveGeneralSettings()
    {
        $general_settings = wpdm_query_var(WPPM_GENERAL_SETTINGS_KEY, 'escs');
        //pd($general_settings);
        update_option(WPPM_GENERAL_SETTINGS_KEY, ($general_settings));
        if (wpdm_is_ajax()) {
            wp_send_json(['success' => true,]);
        }
    }


    function saveSettings()
    {
        if (isset($_POST[WPPM_GENERAL_SETTINGS_NONCE]) && wp_verify_nonce($_POST[WPPM_GENERAL_SETTINGS_NONCE])) {
            $this->saveGeneralSettings();
        }
    }


    public function getGeneralSettingValue($key)
    {
        $general_settings = get_option(WPPM_GENERAL_SETTINGS_KEY);
        $value = stripslashes_deep(htmlspecialchars_decode(wpdm_valueof($general_settings, $key, ['validate' => 'html'])));
        if(!$value && isset($this->default_values[$key]))
            $value = $this->default_values[$key];
        return $value;
    }
}
