<?php

namespace WPDM\WPPM\Subscription;

use DateTime;
use Exception;
use WPDM\WPPM\Invoice\Invoice;

if (!defined('ABSPATH')) exit;




class Subscription
{
    /**
     * wppm subscription id
     */
    private $id;

    /**
     * user id
     */
    private $user_id;

    /**
     * plan id
     */
    private $plan_id;

    /**
     * Plan object
     */
    private $plan;

    /**
     * subscription status
     */
    private $subscription_status; //incomplete, active

    /**
     * Expires at (Current Period Expiration)
     * @param DateTime
     */
    private $expires_at;

    /**
     * renewal notification sending date
     *
     */
    private $expiration_notified_at;

    /**
     * payment system id
     */
    private $payment_system_id;

    /**
     * payment system subscription id
     */
    private $payment_system_subscription_id;

    /**
     * payment system subscription data
     * @param jsonstring|null
     */
    private $payment_system_subscription_data;

    /**
     *
     */
    private $created_at;

    /**
     *
     *
     */
    private $modified_at;

    /**
     * payment systemt subscription data parser
     */
    private $_parser;



    /**
     * constructor
     *
     * this object can be created by passing data array
     *
     */
    public function __construct($data = null)
    {
        switch (gettype($data)) {
            case 'NULL':
                break;
            case 'array':
                $this->fromArray($data);
                break;
            default:
        }
    }

    /**
     *
     * @return obj Subscription
     */
    private function fromArray($data = [])
    {
        $this->setId(wpdm_valueof($data, 'id'));

        $this->setUserId(wpdm_valueof($data, 'user_id'));
        $this->setPlanId(wpdm_valueof($data, 'plan_id'));
        $this->setSubscriptionStatus(wpdm_valueof($data, 'subscription_status'));
        $this->setExpiresAt(wpdm_valueof($data, 'expires_at'));
        $this->setExpirationNotifiedAt(wpdm_valueof($data, 'expiration_notified_at'));

        $this->setPaymentSystemId(wpdm_valueof($data, 'payment_system_id'));
        $this->setPaymentSystemSubscriptionId(wpdm_valueof($data, 'payment_system_subscription_id'));
        $this->setPaymentSystemSubscriptionData(wpdm_valueof($data, 'payment_system_subscription_data'));

        $this->setCreatedAt(wpdm_valueof($data, 'created_at'));
        $this->setModifiedAt(wpdm_valueof($data, 'modified_at'));
    }


    /**
     *
     * @return string
     */
    public static function getTableName()
    {
        global $wpdb;
        return $wpdb->prefix . 'wppm_subscriptions';
    }


    /**
     *
     * @return SubscriptionParser
     */
    public function getParser()
    {
        $PaymentMethod = $this->getPaymentSystem();
        if ( $PaymentMethod && method_exists($PaymentMethod, 'getSubscriptionParser')) {
            if (empty($this->_parser)) {
                $this->_parser = $PaymentMethod->getSubscriptionParser();
            }
            $this->_parser->setData($this->getPaymentSystemSubscriptionData());
            return $this->_parser;
        }
    }


    /**
     * Save (create or update) the obj in db
     *
     * @return id|Error
     */
    public function save()
    {
        global $wpdb;
        $subscriptions_table = $wpdb->prefix . 'wppm_subscriptions';

        $values = [
            'user_id' => $this->getUserId(),
            'plan_id' => $this->getPlanId(),
            'expires_at' => $this->getExpiresAt() ? $this->getExpiresAt()->format('Y-m-d H:i:s') : null,
            'expiration_notified_at' => $this->getExpirationNotifiedAt() ? $this->getExpirationNotifiedAt()->format('Y-m-d H:i:s') : null,
            'subscription_status' => $this->getSubscriptionStatus(),
            'payment_system_id' => $this->getPaymentSystemId(),
            'payment_system_subscription_id' => $this->getPaymentSystemSubscriptionId(),
            'payment_system_subscription_data' => maybe_json_encode($this->getPaymentSystemSubscriptionData())
        ];

        $formats = [
            '%d',
            '%d',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s'
        ];

        if ($this->getId()) {
            //update
            $result = $wpdb->update(
                $subscriptions_table,
                $values,
                ['id' => $this->getId()],
                $formats,
                ['%d']
            );
        } else {
            //create
            $result = $wpdb->insert(
                $subscriptions_table,
                $values,
                $formats
            );

            if ($result) {
                $this->id = $wpdb->insert_id;
            }
        }

        if ($result === false) {
            $msg = $wpdb->last_error . ' from ' . $wpdb->last_query;
            throw new Exception($msg);
        }

        do_action(WPPM_ACTION_AFTER_SUBSCRIPTION_SAVED, $this);

        return $this->getId();
    }


    /**
     * Cancel a subscription
     *
     *
     *
     */
    public function cancel()
    {
        $subscription = $this;
        $payment_system = $this->getPaymentSystem();
        $plan = WPPM()->plan_manager->getPlan($subscription->plan_id);
        if ($payment_system && !$plan->isLifeTimePlan())
            $subscription =  $payment_system->cancelSubscription($this);
        $subscription->setSubscriptionStatusCanceled();
        $subscription->setExpiresAt(0);
        $subscription->save();

        do_action(WPPM_ACTION_AFTER_SUBSCRIPTION_CANCELED, $subscription);
    }



    /**
     * Static method
     *
     *
     * @return Subscription|null
     *
     */
    public static function get($sub_id)
    {
        global $wpdb;
        $table = $wpdb->prefix . 'wppm_subscriptions';

        $rows = $wpdb->get_results("SELECT * FROM $table WHERE `id` = {$sub_id}", ARRAY_A);

        if (count($rows)) {
            $row = array_shift($rows);
            return new Subscription($row);
        }

        return null;
    }


    /**
     * Get single Subscription by subscription id
     *
     * @return Subscription
     */
    public static function getByPaymentSystemSubscriptionId($id)
    {
        global $wpdb;
        $table = $wpdb->prefix . 'wppm_subscriptions';

        $rows = $wpdb->get_results("SELECT * FROM $table WHERE `payment_system_subscription_id` = '{$id}'", ARRAY_A);

        if (count($rows)) {
            $row = array_shift($rows);
            return new Subscription($row);
        }

        return null;
    }


    /**
     *
     * @return [Subscription]
     */
    public static function getByExpiresAfterDays($days = 7)
    {
        $projected_day = new DateTime("+{$days} days");

        $start_of_day = clone $projected_day;
        $start_of_day->modify('today');

        $end_of_day = clone $start_of_day;
        $end_of_day->modify('tomorrow');
        $t = $end_of_day->getTimestamp();
        $end_of_day->setTimestamp($t - 1);

        global $wpdb;
        $subscriptions = [];

        $subscription_table = $wpdb->prefix . 'wppm_subscriptions';

        $query = "SELECT * FROM $subscription_table WHERE `expires_at` >= '{$start_of_day->format('Y-m-d H:i:s')}' AND `expires_at` <= '{$end_of_day->format('Y-m-d H:i:s')}'";
        $rows = $wpdb->get_results($query, ARRAY_A);

        if (count($rows) > 0) {
            foreach ($rows as $row) {
                $sub = new Subscription($row);
                array_push($subscriptions, $sub);
            }
        }
        return $subscriptions;
    }


    /**
     *
     * @return Plan
     */
    public function getPlan()
    {
        if (empty($this->plan)) {
            $this->plan =  WPPM()->plan_manager->getPlan($this->getPlanId());
        }

        return $this->plan;
    }



    /**
     *
     * @return array [Invoice]
     */
    public function getInvoices()
    {
        return Invoice::getBySubscriptionId($this->getId());
    }


    /**
     * get wppm subscription id
     *
     * @return int|null
     */
    public function getId()
    {
        return intval($this->id);
    }


    /**
     * set wppm subscription id
     *
     * @return self
     */
    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }


    /**
     * get subscription user id
     *
     * @return int
     */
    public function getUserId()
    {
        return $this->user_id;
    }


    /**
     *
     * @return WP_User
     */
    public function getUser()
    {
        $user = get_user_by('id', $this->getUserId());
        return $user;
    }


    /**
     * set subscription user id
     *
     * @param int user_id
     *
     * @return self
     */
    public function setUserId($user_id)
    {
        $this->user_id = $user_id;
        return $this;
    }



    /**
     *
     * @return string
     */
    public function getSubscriptionStatus()
    {
        return $this->subscription_status;
    }


    /**
     *
     * @return self
     */
    public function setSubscriptionStatus($status)
    {
        $this->subscription_status = $status;
        return $this;
    }


    /**
     *
     * @return self
     */
    public function setSubscriptionStatusActive()
    {
        $this->subscription_status = "ACTIVE";
        return $this;
    }


    /**
     *
     * @return self
     */
    public function setSubscriptionStatusCanceled()
    {
        $this->subscription_status = "CANCELED";
        return $this;
    }


    /**
     *
     * @return self
     */
    public function setSubscriptionStatusInactive()
    {
        $this->subscription_status = "INACTIVE";
        return $this;
    }


    /**
     *
     * @return boolean
     */
    public function isActive()
    {
        return $this->getSubscriptionStatus() == 'ACTIVE';
    }


    /**
     *
     * @return boolean
     */
    public function isCanceled()
    {
        return $this->getSubscriptionStatus() == 'CANCELED';
    }


    /**
     *
     * @return int
     */
    public function getPlanId()
    {
        return $this->plan_id;
    }


    /**
     * @param int plan_id
     *
     * @return self
     */
    public function setPlanId($plan_id)
    {
        $this->plan_id = $plan_id;
        return $this;
    }


    /**
     * Get payment system id: testpay|paypal|stripe
     *
     * @return enum string
     */
    public function getPaymentSystemId()
    {
        return $this->payment_system_id;
    }


    /**
     *
     * @param string payment_system_id
     *
     * @return self
     */
    public function setPaymentSystemId($payment_system_id)
    {
        $this->payment_system_id = $payment_system_id;
        return $this;
    }


    /**
     *
     * @return PaymentSystem|null
     */
    public function getPaymentSystem()
    {
        return WPPM()->payment_system_manager->getPaymentSystem($this->getPaymentSystemId());
    }


    /**
     * Get payment system id: testpay|paypal|stripe
     *
     * @return enum string
     */
    public function getPaymentSystemSubscriptionId()
    {
        return $this->payment_system_subscription_id;
    }


    /**
     *
     * @param string payment_system_id
     *
     * @return self
     */
    public function setPaymentSystemSubscriptionId($id)
    {
        $this->payment_system_subscription_id = $id;
        return $this;
    }


    /**
     *
     * @return jsondecode|null
     */
    public function getPaymentSystemSubscriptionData()
    {
        return maybe_json_decode($this->payment_system_subscription_data, true);
    }


    /**
     * @param any data
     *
     * @return self
     */
    public function setPaymentSystemSubscriptionData($data)
    {
        $this->payment_system_subscription_data = maybe_json_encode($data);

        if ($this->getParser()) {
            $this->setExpiresAt($this->getParser()->getCurrentPeriodEndTime());
        }
        return $this;
    }


    /**
     * first get the subscription data from the payment system server
     * then set the data
     *
     * @return self
     */
    public function fetchPaymentSystemSubscriptionData()
    {
        if ($this->getPaymentSystem()->isRemotePaymentSystem()) {
            $subscription_data = $this->getPaymentSystem()->getSubscriptionDetails($this->getPaymentSystemSubscriptionId());

            $this->setPaymentSystemSubscriptionData($subscription_data);
        }

        return $this;
    }


    /**
     *
     * @return wp_date formated
     */
    public function getNextBillingDate()
    {
        return $this->getParser() ? wp_date(get_option('date_format'), $this->getParser()->getNextBillingTime()) : 'NA';
    }


    /**
     *
     * @return timestamp
     */
    public function getCurrentPeriodEndTime()
    {
        return $this->getParser() ?  $this->getParser()->getNextBillingTime()  : null;
    }


    /**
     * Get expires at (Current Period Expiration)
     *
     * @return DateTime|null
     */
    public function getExpiresAt()
    {
        return wppm_get_date_time($this->expires_at);
    }


    /**
     * Returns false If current date is over the 2 days than expires at date
     *
     * @return boolean
     */
    public function isExpired()
    {
        if ($this->getExpiresAt()) {
            $check_date = new DateTime('+ 2 days');
            if ($check_date > $this->getExpiresAt()) {
                return true;
            }
        }

        return false;
    }


    /**
     * Set expires at (Current Period Expiration)
     * @param int|string
     *
     * @return  self
     */
    public function setExpiresAt($expires_at)
    {
        $this->expires_at = $expires_at;

        return $this;
    }


    /**
     *
     * @return wp_date format
     */
    public function getFormattedExpiresDate()
    {
        return $this->getExpiresAt() ? wp_date(get_option('date_format'), $this->getExpiresAt()->getTimestamp()) : 'NA';
    }


    /**
     * @param string|timestamp
     *
     * @return self
     */
    public function setExpirationNotifiedAt($time)
    {
        $this->expiration_notified_at = $time;
        return $this;
    }


    /**
     *
     * @return DateTime|null
     */
    public function getExpirationNotifiedAt()
    {
        return wppm_get_date_time($this->expiration_notified_at);
    }


    /**
     *
     * @return url
     */
    public function getCancelUrl()
    {
        return add_query_arg([
            'wppm_task' => 'CANCEL_SUBSCRIPTION',
            'subscription_id' => $this->getId(),
            'redirect' => urlencode($_SERVER['REQUEST_URI']),
        ]);
    }


    /**
     *
     * @return DateTime
     */
    public function getCreatedAt()
    {
        return wppm_get_date_time($this->created_at);
    }


    /**
     *
     * @return self
     */
    public function setCreatedAt($date)
    {
        $this->created_at = $date;
        return $this;
    }

    /**
     *
     * @return DateTime
     */
    public function getModifiedAt()
    {
        return wppm_get_date_time($this->modified_at);
    }

    /**
     *
     * @return self
     */
    public function setModifiedAt($date)
    {
        $this->modified_at = $date;
        return $this;
    }


    /**
     *
     * @return double
     */
    public function getTotalInvoicesAmount()
    {
        $amount = 0;
        $invoices = $this->getInvoices();
        if (is_array($invoices)) {
            foreach ($invoices as $invoice) {
                if (is_object($invoice))
                    $amount += $invoice->getAmount();
            }
        }

        return $amount;
    }
}
