<?php

namespace WPDM\WPPM\Subscription;

use DateTime;
use Exception;
use WP_User;

if (!defined('ABSPATH')) exit;


class SubscriptionManager
{

    /**
     * holds singleton instance of the class
     */
    private static $instance;


    /**
     * subscriptions mem cache
     */
    private $_subscriptions;


    /**
     * get the singleton instance of the class
     */
    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }

        return self::$instance;
    }


    /**
     * private constructor of the class
     * so this class cannot be created outside
     *
     */
    private function __construct()
    {
        $this->loadDependencies();
        $this->addHooks();
    }


    /**
     * Load the dependency files
     *
     */
    private function loadDependencies()
    {
    }


    /**
     * Add necessary hooks
     *
     */
    public function addHooks()
    {
        add_action('init', array($this, 'actionTask'));
        add_action('wp_login', array($this, 'checkUserSubscriptionsValidity'), 10, 2);
    }



    /**
     *
     * @return string
     */
    public static function getTableName()
    {
        global $wpdb;
        return $wpdb->prefix . 'wppm_subscriptions';
    }


    /**
     * List all subscribers
     */
    public function listSubscribers()
    {
        $args = [];
        $file = __DIR__ . '/list-subscribers.php';
        wppm_render_template($file, $args);
    }


    /**
     * Get all the subscriptions of a user
     *
     * @return array [Subscription]
     */
    public function getSubscriptionsOfUser($user_id)
    {
        global $wpdb;
        $subscriptions = [];

        $subscription_table = $wpdb->prefix . 'wppm_subscriptions';

        $rows = $wpdb->get_results("SELECT * FROM $subscription_table WHERE `user_id` = {$user_id} ORDER BY `modified_at` DESC ", ARRAY_A);

        if (count($rows) > 0) {
            foreach ($rows as $row) {
                $sub = new Subscription($row);
                array_push($subscriptions, $sub);
            }
        }
        return $subscriptions;
    }


    /**
     * Get all subscriptions of current user
     *
     * @return array
     */
    public function getSubscriptionsOfCurrentUser()
    {
        $user_id = get_current_user_id();
        if (empty($user_id)) return [];

        return $this->getSubscriptionsOfUser($user_id);
    }


    /**
     * Get active subscriptions of a user
     *
     * @return array [Subscription]
     */
    public function getActiveSubscriptionsOfUser($user_id)
    {
        $subscriptions = $this->getSubscriptionsOfUser($user_id);

        $filtered = array_filter($subscriptions, function ($s) {
            return $s->isActive();
        });

        return array_values($filtered);
    }

    /**
     * Get active subscriptions of a user
     *
     * @return array [Subscription]
     */
    public function getActivePlans($user_id)
    {
        $subscriptions = $this->getSubscriptionsOfUser($user_id);

        $filtered = array_filter($subscriptions, function ($s) {
            return $s->isActive();
        });

        $subscriptions =  array_values($filtered);
        $plans = [];
        foreach ($subscriptions as $subscription) {
            $plans[] = "wppm_".$subscription->getPlanId();
        }
        return $plans;
    }


    /**
     *
     * @return [Subscription]
     */
    public function getActiveSubscriptionsOfCurrentUser()
    {
        $user_id = get_current_user_id();
        if (empty($user_id)) return [];

        return $this->getActiveSubscriptionsOfUser($user_id);
    }


    /**
     *
     * @return array [plan id]
     */
    public function getPlanIdsOfUser($user_id)
    {
        $plan_ids = [];

        $subscriptions = $this->getSubscriptionsOfUser($user_id);

        foreach ($subscriptions as $s) {
            array_push($plan_ids, $s->getPlanId());
        }
        return $plan_ids;
    }

    /**
     *
     * @return Plan[]
     */
    public function getActivePlansOfUser($user_id)
    {
        $plans = [];

        foreach ($this->getActiveSubscriptionsOfUser($user_id) as $s) {
            if ($s->getPlan()) array_push($plans, $s->getPlan());
        }
        return $plans;
    }


    /**
     *
     * @return array [plan id]
     */
    public function getActivePlanIdsOfUser($user_id)
    {
        $plan_ids = [];

        $subscriptions = $this->getActiveSubscriptionsOfUser($user_id);

        foreach ($subscriptions as $s) {
            array_push($plan_ids, $s->getPlanId());
        }
        return $plan_ids;
    }


    /**
     *
     * @return array [plan id]
     */
    public function getPlanIdsOfCurrentUser()
    {
        $user_id = get_current_user_id();
        if (empty($user_id)) return [];

        return $this->getPlanIdsOfUser($user_id);
    }


    /**
     *
     *  @return array [plan id]
     */
    public function getActivePlanIdsOfCurrentUser()
    {
        $user_id = get_current_user_id();
        if (empty($user_id)) return [];

        return $this->getActivePlanIdsOfUser($user_id);
    }


    /**
     * check if current user subscribed to the given plan
     *
     * @return boolean
     */
    public function currentUserSubscribedTo($plan_id)
    {
        return in_array($plan_id, $this->getActivePlanIdsOfCurrentUser());
    }


    /**
     * save subscription into the database
     *
     * after save add role to the user
     *
     * @return int subscription id
     */
    public function saveSubscription($data)
    {
        try {
            $sub = new Subscription($data);
            // pd($sub);
            $s_id =  $sub->save();
            return $s_id;
        } catch (Exception $e) {
            return $e->getMessage();
        }
    }


    /**
     * action task
     *
     */
    public function actionTask()
    {
        $wppm_task = wpdm_query_var('wppm_task');
        if ($wppm_task && $wppm_task == 'CANCEL_SUBSCRIPTION') {

            $subscription_id = wpdm_query_var('subscription_id');
            try {
                $subscription = Subscription::get($subscription_id);
                if ($subscription && $subscription->getUserId() == get_current_user_id()) {
                    $subscription->cancel();
                    wp_safe_redirect(urldecode(wpdm_query_var('redirect', 'escs')));
                    die();
                }
            } catch (Exception $e) {
                wp_safe_redirect(urldecode(wpdm_query_var('redirect', 'escs')));
                die();
                //p($e->getMessage());
            }
        }
    }


    /**
     * Checks the user active subscriptions expires date
     * Make subscriptions Canceled if current date exceeds by 2+ days the expires date
     *
     */
    public function checkUserSubscriptionsValidity($username, WP_User $user)
    {
        $active_subscriptions = $this->getActiveSubscriptionsOfUser($user->ID);
        $wppm_roles = WPPM()->role_manager->getRoleIds();

        $wppm_roles = array_intersect($user->roles, $wppm_roles);
        $user_roles = [];

        foreach ($active_subscriptions as $subscription) {
            $user_roles[] = "wppm_".$subscription->getPlanId();
            try {
                if ($subscription->isExpired()) {
                    //fetch subscription data from remote
                    $subscription->fetchPaymentSystemSubscriptionData();

                    //if subscription still expired then make it canceled
                    if ($subscription->isExpired()) {
                        $subscription->setSubscriptionStatusCanceled();
                    } elseif ($subscription->isActive()) {
                        $subscription->setSubscriptionStatusActive();
                    }

                    $subscription->save();
                }
            } catch (Exception $e) {
                wppm_write_log($e->getMessage());
            }
        }
        foreach ($wppm_roles as $role) {
            if(!in_array($role, $user_roles)) {
                $user->remove_role($role);
            }
        }
    }


    /**
     *
     * @return string
     */
    private function applyQueryFilters($query, SubscriptionFilter $filter = null)
    {
        if ($filter) {
            if ($filter->getSubscriptionStatus()) {
                $query  .= " AND `subscription_status` = '{$filter->getSubscriptionStatus()}' ";
            }

            if ($filter->getPlanId()) {
                $query  .= " AND `plan_id` = '{$filter->getPlanId()}' ";
            }

            $date_field = 'created_at';
            if ($filter->getSubscriptionStatus() == 'CANCELED') {
                $date_field = 'modified_at'; // we are assuming cancellation date is the modified date
            }

            if ($filter->getFromDate()) {
                $query  .= " AND `{$date_field}` >= '{$filter->getFromDate()->format('Y-m-d H:m:i')}' ";
            }

            if ($filter->getToDate()) {
                $query  .= " AND `{$date_field}` <= '{$filter->getToDate()->format('Y-m-d H:m:i')}' ";
            }
        }

        return $query;
    }



    /**
     *
     * @return Subscription[]
     */
    public function getSubscriptions(SubscriptionFilter $filter = null)
    {
        global $wpdb;
        $table_name =  self::getTableName();

        $query = "SELECT * FROM {$table_name} WHERE 1 ";

        $query = $this->applyQueryFilters($query, $filter);

        $rows = $wpdb->get_results($query, ARRAY_A);
        $subscriptions = [];
        if (count($rows) > 0) {
            foreach ($rows as $row) {
                $sub = new Subscription($row);
                array_push($subscriptions, $sub);
            }
        }
        return $subscriptions;
    }


    /**
     *
     *
     * @return int
     */
    public function countSubscriptions(SubscriptionFilter $filter = null)
    {
        global $wpdb;
        $table_name =  self::getTableName();

        $query = "SELECT COUNT(*) FROM {$table_name} WHERE 1 ";

        $query = $this->applyQueryFilters($query, $filter);

        return (int) $wpdb->get_var($query);
    }


    /**
     *
     * @return int[] [21, 20, 21, ....] 12 integer values
     */
    public function countMonthlySubscriptions(SubscriptionFilter $filter)
    {
        $year = $filter->getFromDate() ? $filter->getFromDate()->format('Y') : date('Y');

        $counts = [];
        foreach (wppm_get_all_months() as $month) {
            $filter->setIntervalMonth($month['value'], $year);

            array_push($counts, $this->countSubscriptions($filter));
        }

        return $counts;
    }


    /**
     *
     * @return int[] [ 01 => 3, 02 => 1, 03 => 7,...]
     */
    public function countDailySubscriptions(SubscriptionFilter $filter)
    {
        $start_day = $filter->getFromDate()->format('d');
        $end_day = $filter->getToDate()->format('d');
        $data = [];
        for ($i = $start_day; $i <= $end_day; $i++) {
            $data[sprintf("%02d", $i)] = 0;
        }

        $subscriptions = $this->getSubscriptions($filter);

        foreach ($subscriptions as $sub) {

            if ($filter->getSubscriptionStatus() == 'CANCELED') {
                $day = $sub->getModifiedAt()->format('d');
            } else {
                $day = $sub->getCreatedAt()->format('d');
            }

            $data[$day] = $data[$day] + 1;
        }
        return $data;
    }


    /**
     *
     * @return int
     */
    public function countActiveSubscriptions($filter = null)
    {
        $filter = $filter ?: new SubscriptionFilter();
        $filter->setSubscriptionStatus('ACTIVE');

        return $this->countSubscriptions($filter);
    }


    /**
     *
     * @return int
     */
    public function countCanceledSubscriptions($filter = null)
    {
        $filter = $filter ?: new SubscriptionFilter();
        $filter->setSubscriptionStatus('CANCELED');

        return $this->countSubscriptions($filter);
    }



    /**
     *
     * @return double
     */
    public function getCancellationRate()
    {
        $count_all = $this->countSubscriptions();
        if ($count_all == 0)  return 0.00;

        $count_active = $this->countActiveSubscriptions();
        return number_format((($count_all - $count_active) / $count_all) * 100, 2);
    }
}
