<?php

namespace WPDM\WPPM;

if (!defined('ABSPATH')) die();

use WPDM\WPPM\Access\AccessManager;
use WPDM\WPPM\Account\AccountManager;
use WPDM\WPPM\Auth\AuthManager;
use WPDM\WPPM\Checkout\CheckoutManager;
use WPDM\WPPM\Core\AlertManager;
use WPDM\WPPM\Core\InstallManager;
use WPDM\WPPM\Coupon\CouponManager;
use WPDM\WPPM\Cron\CronManager;
use WPDM\WPPM\Email\EmailManager;
use WPDM\WPPM\Invoice\InvoiceManager;
use WPDM\WPPM\Payment\PaymentSystemManager;
use WPDM\WPPM\Payment\WebhookListener;
use WPDM\WPPM\Plan\PlanManager;
use WPDM\WPPM\Role\RoleManager;
use WPDM\WPPM\Settings\SettingsManager;
use WPDM\WPPM\Stats\StatsManager;
use WPDM\WPPM\Subscription\AccessControl;
use WPDM\WPPM\Subscription\SubscriptionManager;
use WPDM\WPPM\User\UserManager;



/**
 * Root Class of the pluging
 * This class helps to initiate and hold other necessary classes
 *
 */
class WPPM
{
    /**
     * holds singleton instance of the class
     */
    private static $instance;

    /**
     * holds the alert manager object
     */
    public $alert_manager;

    /**
     * holds the install_manager object
     */
    public $install_manager;

    /**
     * holds the Settings_Manager onject
     */
    public $settings_manager;

    /**
     * @class \WPDM\WPPM\Plan\PlanManager
     */
    public $plan_manager;

    /**
     * holds the email manager
     */
    public $email_manager;

    /**
     * holds User Manager object
     *
     */
    public $user_manager;

    /**
     * holds the Auth Manager object
     */
    public $auth_manager;

    /**
     * holds the Role Manager object
     */
    public $role_manager;

    /**
     * holds Payment_Manager object
     */
    public $payment_system_manager;

    /**
     *
     */
    public $payment_webhook_listener;

    /**
     * Subscription Manager object
     */
    public $invoice_manager;

    /**
     * Subscription Manager object
     */
    public $subscription_manager;

    /**
     * Checkout Manager object
     *
     */
    public $checkout_manager;

    /**
     * Checkout Manager object
     *
     */
    public $account_manager;


    /**
     * private constructor
     * loads dependencies and hooks
     *
     */
    private function __construct()
    {
        $this->loadDependencies();
        $this->addHooks();
    }


    /**
     * get singleton instance
     *
     */
    public static function getInstance()
    {
        if (!self::$instance) {
            self::$instance = new self;
        }

        return self::$instance;
    }


    /**
     * Load the dependency files
     *
     */
    private function loadDependencies()
    {

        $this->alert_manager = AlertManager::getInstance();

        $this->install_manager = InstallManager::getInstance();

        $this->cron_manager = CronManager::getInstance();

        $this->settings_manager = SettingsManager::getInstance();

        $this->plan_manager = PlanManager::getInstance();

        $this->email_manager = EmailManager::getInstance();

        $this->user_manager = UserManager::getInstance();

        $this->auth_manager = AuthManager::getInstance();

        $this->payment_system_manager = PaymentSystemManager::getInstance();

        $this->payment_webhook_listener = WebhookListener::getInstance();

        $this->invoice_manager = InvoiceManager::getInstance();

        $this->subscription_manager = SubscriptionManager::getInstance();

        $this->access_control = AccessControl::getInstance();

        $this->role_manager = RoleManager::getInstance();

        $this->checkout_manager = CheckoutManager::getInstance();

        $this->account_manager = AccountManager::getInstance();

        $this->stats_manager = StatsManager::getInstance();

        $this->coupon_manager = CouponManager::getInstance();

        $this->access_manager = AccessManager::getInstance();

        global $wpmp_currency_symbol, $wpmp_currency_code;
        $wpmp_currency_symbol = function_exists('wpdmpp_currency_sign') ? wpdmpp_currency_sign() : '$';
        $wpmp_currency_code = function_exists('wpdmpp_currency_code') ? wpdmpp_currency_code() : 'USD';
    }


    /**
     * Called during plugin activation
     *
     */
    public function activate()
    {
        $this->install_manager->activate();
        $this->cron_manager->scheduleCronEvents();
    }


    /**
     * Called during plugin deavtivation
     *
     */
    public function deactivate()
    {
        $this->cron_manager->clearCronEvents();
    }


    /**
     *  Add actions,filter, shortcode hooks
     *
     */
    public function addHooks()
    {
        add_action('init', array($this, 'actionWPPMTask'));
        add_filter('update_plugins_wpdm-archive-page', [$this, "updatePlugin"], 10, 4);
        if (is_admin()) {

            add_action('admin_menu', array($this, 'add_admin_menus'));
            add_action('admin_enqueue_scripts', array($this, 'add_admin_styles_and_scripts'));
        }

        add_action("wp_enqueue_scripts", array($this, "add_public_styles_and_scripts"));
    }


    /**
     * Add public styles and scripts
     *
     */
    public function add_public_styles_and_scripts()
    {

        wp_enqueue_style('wppm', WPPM_PUBLIC_URL . 'css/style.css');
        wp_enqueue_script('jquery');
        wp_enqueue_script('jquery-form');
    }


    /**
     * Enqueue admin styles and scripts
     *
     */
    public function add_admin_styles_and_scripts($hook)
    {

        if (wpdm_query_var('post_type') === 'wppm_plan' || get_post_type(wpdm_query_var('post')) === 'wppm_plan' || $hook === 'profile.php') {

            wp_enqueue_script('jquery-ui-datepicker');
            wp_enqueue_script('jquery-form');
            wp_enqueue_script('wpdm-admin', WPPM_PUBLIC_URL . 'js/admin.js');

            wp_enqueue_style('wpdm-fontawesome', WPDM_FONTAWESOME_URL);

            wp_enqueue_style('wpdm-admin-bootstrap');
            wp_enqueue_script('wpdm-admin-bootstrap');

            wp_enqueue_style('wppm-admin', WPPM_PUBLIC_URL . 'css/admin-styles.css');
            wp_enqueue_style('data-tables', WPPM_PUBLIC_URL . 'css/data_tables.css');
        }
    }


    /**
     * Add admin menus
     *
     */
    public function add_admin_menus()
    {
        // Add settings page under Plan custom post type menu
        add_submenu_page(
            'edit.php?post_type=wppm_plan',
            "Membership - " . __('Subscribers', "wppromembership"),
            __('Subscribers', WPPM_TEXTDOMAIN),
            'manage_options',
            'wppm_subscribers',
            array($this->subscription_manager, 'listSubscribers')
        );
        add_submenu_page(
            'edit.php?post_type=wppm_plan',
            "Membership - " . __('Coupons', "wppromembership"),
            __('Coupons', WPPM_TEXTDOMAIN),
            'manage_options',
            'wppm_coupons',
            array($this->coupon_manager, 'renderCouponsPage')
        );
        add_submenu_page(
            'edit.php?post_type=wppm_plan',
            "Membership - " . __('Stats', "wppromembership"),
            "Statistics",
            'manage_options',
            'wppm_stats',
            array($this->stats_manager, 'renderStatsPage')
        );
        add_submenu_page(
            '',
            "Membership - " . __('WPPM Account', "wppromembership"),
            "WPPM Account",
            'manage_options',
            'wppm_account',
            array($this->account_manager, 'renderUserAccount')
        );
        add_submenu_page(
            'edit.php?post_type=wppm_plan',
            "Membership - " . __('Settings', "wppromembership"),
            "Settings",
            'manage_options',
            'wppm_settings',
            array($this->settings_manager, 'renderGlobalSettingsPage')
        );

    }


    public function actionWPPMTask()
    {
        $task = wpdm_query_var('wppm_task');
        if ($task && $task === 'SEND_SUBSCRIPTION_EXPIRATION_REMINDERS') {
            do_action(WPPM_EVENT_SEND_SUBSCRIPTION_EXPIRATION_REMINDERS);
        }
    }

    function updatePlugin($update, $plugin_data, $plugin_file, $locales){
        $id = 'wppromembership';
        $latest_versions = WPDM()->updater->getLatestVersions();
        $latest_version = wpdm_valueof($latest_versions, $id);
        $access_token = wpdm_access_token();
        $update = [];
        $update['id']           = $id;
        $update['slug']         = $id;
        $update['url']          = $plugin_data['PluginURI'];
        $update['tested']       = true;
        $update['version']      = $latest_version;
        $update['package']      = $access_token !== '' ? "https://www.wpdownloadmanager.com/?wpdmpp_file={$id}.zip&access_token={$access_token}" : '';
        return $update;
    }
}
